c****************subroutine QQVV *************************************
c
c	Vera Hankele, <vera@particle.physik.uni-karlsruhe.de>
c	Initial version:  2006 April
c	Last modified:    
C
C  QQWW calculates the matrix elements**2 for qq -> WW -> ve e+ mu- vm~
C
C        q1 q2    ---->   W+ W-,   W+ ---> f4-bar f3, W- ---> f6-bar f5
C
C  and crossing related processes. Pauli interference terms for
c  identical fermions are neglected.
c
C  This code is modified to allow for virtual corrections, more precisely
C  the interference of Born with the finite part of virtual diagrams
C  for 
c
c  INPUT:  NLO = 1       return uuww = |M_born|^2 + 2Re(M_Born^* M_virt)
c          NLO = 0       return uuww = |M_born|^2   etc.
c     
c          N = 1         Momentum configuration 1 (LO)
c          N = 2         Momentum configuration 2: emission of 1 line
c          N = 3         Momentum configuration 3: emission of 2 line
c
c*********************************************************************

      subroutine qqVV(pbar,sign, nlo, N, uuww, ddww)

      implicit none

#include "VBFNLO/utilities/global.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/coupl.inc"


c     electroweak couplings are taken from KOPPLN

      double precision  clr, xm2, xmg, b, v, a
      COMMON /BKOPOU/   CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),
     1                  V(4,5),A(4,5)
      double precision  pbar(0:3,2+n_v)
      double precision  uuww, ddww, res(2),resv(2)
      double precision  p(0:3,2+n_v), p21(0:4),
     1                  fqp(0:4,2), fqm(0:4,2)
      integer  sign(2+n_v), nlo, mu, i, j, jj, k, kk, id,
     1         isig, isig1, N
      integer  ifl(2,2), js1, js3
      double complex psiwp(2,2), psiwm(2,2),
     1               jwp(0:5,-1:1), jwm(0:5,-1:1),
     1               jvwp(0:5,-1:1), jvwm(0:5,-1:1)
      double complex temp, temp1
      double complex mat(2,-1:1,3), matv(2,-1:1,3)
      double complex madgr(3,-1:1,2)
      double complex mm(2,-1:1), mv(2,-1:1), zero
      double complex ma, mz, m1, m2
      double complex mv1,mv2
      double complex psi(2,-1:1,4), jqq(0:5,-1:1,2)
      double complex dotcc, s1c
      double precision dotp
      external dotcc, s1c, dotp
      save ifl
      parameter (zero = (0d0,0d0) )

c variables for virtual corrections

      double precision c2,c2o4pi
      parameter (c2=4d0/3d0, c2o4pi=c2/4d0/pi)
      logical lnlo, lbox, ldebug, ldebugm, linit
      data linit /.true./
      save linit
      parameter(ldebugm = .false., ldebug=.false.)

c variables for test purposes

      double precision test
      logical veratest, boxtest
      parameter (veratest=.false., boxtest=.false.)

      lnlo = NLO.ne.0       ! include some virtual stuff if T
      lbox = NLO.eq.-4.or.NLO.eq.1      ! call boxline if T

c define flavors of external quarks for the 2 NC subprocesses

      if (linit) then
         linit = .false.
         k = 1                  ! uuww
         ifl(1,k) = 3
         ifl(2,k) = 3
         k = 2                  ! ddww
         ifl(1,k) = 4
         ifl(2,k) = 4
      endif

      do k = 1,2
         do isig1 = -1,1,2
            do i = 1,3
                  mat(k,isig1,i) = 0
                  matv(k,isig1,i) = 0
            enddo
         enddo
      enddo

c define the internal momenta

      do mu = 0,3
         do i = 1,2+n_v
            p(mu,i) = pbar(mu,i)*sign(i)
         enddo
      enddo

c get the external quark spinors (including factor sqrt(2E) )

      call psi0m(2,pbar(0,1),sign(1),psi)

c------- T-CHANNEL ----------------------------------------------
   
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qp(0,N),wp(1,N),
     &              psiwp(1,2),fqp(0,2))
      call bra2c(psi(1,-1,2),.true.,p(0,2),-1,qm(0,N),wm(1,N),
     &              psiwm(1,2),fqm(0,2))

      call curr6(-1,psiwp(1,2),fqp(0,2),
     &              psi(1,-1,1),p(0,1), jwp(0,-1) )
      call curr6(-1,psiwm(1,2),fqm(0,2),
     &              psi(1,-1,1),p(0,1), jwm(0,-1) )

      if (lbox.and.(.not.veratest)) then
            call boxlineVV(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2), !uuWW
     1                    .true., wm(1,N), 1,   jwm(0,-1), 
     2                    jvwm(0,-1) )   
            call boxlineVV(-1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2), !ddWW
     1                    .true., wp(1,N), 1,   jwp(0,-1), 
     2                    jvwp(0,-1) )
      endif

      m1 = dotcc(jwm(0,-1),wp(1,N))   
      m2 = dotcc(jwp(0,-1),wm(1,N))
      mat(1,-1,1) = clr(3,3,-1)**2 * m1
      mat(2,-1,1) = clr(3,3,-1)**2 * m2

      if (lbox) then
         mv1 = dotcc(jvwm(0,-1),wp(1,N))
         mv2 = dotcc(jvwp(0,-1),wm(1,N))

         matv(1,-1,2) = clr(3,3,-1)**2 * mv1  ! uuWW
         matv(2,-1,2) = clr(3,3,-1)**2 * mv2  ! ddWW
      endif

c------- end of T-CHANNEL computation

c-------  S-CHANNEL --------------------------------------

c get the f-fbar currents J21^mu=jqq(mu,*,1) 

      call curr6(1,psi(1,-1,2),p(0,2),psi(1,-1,1),p(0,1),jqq(0,-1,1))

c contract with atoww/ztoww tensor

      do isig1 = -1,1,2
         ma = -dotcc(aww(0,N),jqq(0,isig1,1))
         mz = -dotcc(zww(0,N),jqq(0,isig1,1))
         do k = 1,2
            mat(k,isig1,2) = ma*clr(ifl(1,k),1,isig1)
            mat(k,isig1,3) = mz*clr(ifl(1,k),2,isig1)
            matv(k,isig1,1) = (0d0,0d0)
         enddo
      enddo

c-------- end of S-CHANNEL computation ---------------------------

c------------------------------------------------------
c comparison with madgraph

      if (ldebugm) then

         do isig = -1,1,2
            do i = 1,3
               do k = 1,2

                  madgr(i,isig,k) = 0d0

               enddo
            enddo
         enddo

         do isig = -1,1,2
            do k = 1,2

         madgr(1,isig,k) = tamp(1,isig,k)
         madgr(2,isig,k) = tamp(2,isig,k) + tamp(5,isig,k)
     &                   + tamp(7,isig,k)
         madgr(3,isig,k) = tamp(3,isig,k) + tamp(4,isig,k)
     &                   + tamp(6,isig,k) + tamp(8,isig,k)
     &                   + tamp(9,isig,k)

            enddo
         enddo

c print results from checks:

         do isig = -1,1,2
            do k = 1,2
               do i=1,3

    	    print*,"ratio for k = ",k," isig = ",isig," i = ",i,":",
     &	    	mat(k,isig,i)/madgr(i,isig,k)
            print*,"mat(",k,",",isig,",",i,") = ",mat(k,isig,i)
            temp = dreal(mat(k,isig,i))**2+dimag(mat(k,isig,i))**2
            print*,temp
            print*,"madgr(",i,",",isig,",",k,") = ",madgr(i,isig,k)
            temp1 = dreal(madgr(i,isig,k))**2+dimag(madgr(i,isig,k))**2
            print*,temp1
            print*,"temp/temp1 = ",temp/temp1
            print*," "

               enddo
            enddo
         enddo

      endif ! end of debugging block

c------------------------------------------------------


c sum the graphs, square them and map them onto uuww and ddww

      do k = 1,2
         res(k) = 0
         resv(k) = 0
         do isig1 = -1,1,2
            mm(k,isig1) = 0
            do i = 1,3
               mm(k,isig1) = mm(k,isig1) +  mat(k,isig1,i)
            enddo
            res(k)= res(k) + dreal(mm(k,isig1))**2
     &                     + dimag(mm(k,isig1))**2

c  add Born type term and multiply by F_q = alphas*C_2/4pi
c  the additional factor for the born term is after adding the 
c  subtraction term and the counter term for the renormalization
c  of the pdfs (see also D(x) in m2s_qqVVj.f)

            if (lnlo) then
               mv(k,isig1) = +  matv(k,isig1,2)  !virtual not prop. to born
               if (veratest) then
                  mv(k,isig1) = 0d0
               endif
            endif
            if (nlo.gt.0) then
               mv(k,isig1) = als(1,1)*c2o4pi*
     1                  (mv(k,isig1) + mm(k,isig1)* cvirtVV) 
            else if(nlo.lt.0) then
               mv(k,isig1) = als(1,1)*c2o4pi*mv(k,isig1)
            else
                mv(k,isig1) = (0d0, 0d0)
            endif

           resv(k) = resv(k) + 2d0*dreal( mm(k,isig1)   *
     1              conjg( mv(k,isig1) ) )

         enddo

         if (veratest) then
            if (nlo.eq.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif
         else
            if (nlo.ge.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif  
         endif
      enddo

      uuww = res(1)
      ddww = res(2)

      return
      end


c----------  boxlineVV(isigmax, ... ) ----------------------------------------
c
      subroutine boxlineVV(isigmax, psi1,psi2,k1,k2,
     1                    ld0calc, jext, ivmu, 
     2                    jborn,    jvirt)
      implicit none
      integer isigmax, ivmu
      logical ld0calc
      double precision k1(0:3),k2(0:3)
      double complex psi1(2,-1:1),psi2(2,-1:1), jext(0:5), 
     1               jborn(0:5,-1:1),jvirt(0:5,-1:1)
c
c  variant of the boxline routines which returns the virtual corrections 
c  without contraction of one of the external gauge bosons, i.e. in form of a
c  current. Also, results for both fermion chiralities are returned for isgmax = +1
c 
c  determine the finite virtual corrections to the fundamental box-type
c  graph, i.e. the sum of all virtual corrections to the Born diagram
c
c    psi1(k1)    ---->---------->-------------->--  psi2(k2)
c                         S              S
c                         S              S
c                         S              S
c                     q1  V           q2 V
c                         S              S
c                         S              S
c
c                        mu           jext(q2)      for ivmu = 1 
c                     jext(q1)          mu          for ivmu = 2
c
c          ====       eps1(q1)        eps2(q2)
c
c   note: k1 = k2+q1+q2, k1.k1 = 0 = k2.k2, q1.q1,q2.q2 arbitrary
c
c  INPUT:      isigmax         -1: only Lefthanded fermions, +1: return L and R
c              psi1(2,-1:1)    the incoming Weyl spinor for both chiralities
c              psi2(2,-1:1)    the outgoing Weyl spinor, psi^+(k2)
c       k1(0:3),k2(0:3)        the corresponding momenta in direction of
c                              the fermion line
c       ld0calc                call BCD_fill_c if true
c       jext(0:3)              specified polarization vector/current" 
c                              corresponding to 1 external EW boson
c       jext(4:5)              outflowing momenta of this EW bosons
c                              q1(0:3) or q2(0:3) in DHELAS notation
c       jborn(0:5,-1:1)        the current of free index at ivmu for the 
c                              corresponding Born diagram and 2 quark chiralities
c
c  OUTPUT:   jvirt(0:5,-1:1)   the amplitude for the finite part of all 
c                              virtual 1-loop QCD corrections for free index at ivmu
c
      logical ldebug
      parameter (ldebug=.false.)

#include "VBFNLO/utilities/scales.inc"

      double precision pi
      parameter (pi = 3.141592653589793d0)

common block for D_ij etc. information via bcd_fill_c
      double complex D0c, Dijc(3,13), Teps1c, Teps2c, Tbornc,Tg1c,Tg2c
      double precision sc, tc, q1sqc, q2sqc
      common /bcd_qqv_c/ sc,tc,q1sqc,q2sqc, D0c,Dijc, 
     1                   Teps1c,Teps2c,Tbornc,Tg1c,Tg2c
C   local variables
      integer mu, nu, isig, icount
      data icount /0/
      save icount
      double precision s,t,mom(0:3),dotp
      external dotp
      double precision q1(0:4),q2(0:4), temp
      double complex eps1(0:3),eps2(0:3),mborn,mvirt
      double complex me1, me2, mq, d0t, s1c, s1r, sc3, z
      external d0t, s1c, s1r, sc3
      double complex ce1,ce2,cq,cborn
      double complex d0, d11,d12,d13, d21,d22,d23,d24,d25,d26,d27
      double complex d31,d32,d33,d34,d35,d36,d37,d38,d39,d310,
     1               d311,d312,d313
      double complex eps1k1, eps1k2, eps1q2, eps1eps2, 
     1               eps2k1, eps2q1, eps2k2, eps1q1, eps2q2,
     2               gce1, gce2, gcq


c transfer momentum information of EW bosons to q1,q2
      if (ivmu.eq.1) then   ! eps2 and q2 are specified, eps1 = mu = free index
         q2(0) = dreal(jext(4))
         q2(1) = dreal(jext(5))
         q2(2) = dimag(jext(5))
         q2(3) = dimag(jext(4))
         do mu = 0,3
            eps2(mu) = jext(mu)
            mom(mu) = k2(mu)+q2(mu)
            q1(mu) = k1(mu)-mom(mu)
         enddo
   
      elseif (ivmu.eq.2) then     ! eps1 and q1 are specified, eps2 = mu = free index
         q1(0) = dreal(jext(4))
         q1(1) = dreal(jext(5))
         q1(2) = dimag(jext(5))
         q1(3) = dimag(jext(4))
         do mu = 0,3
            eps1(mu) = jext(mu)
            mom(mu) = k1(mu)-q1(mu)
            q2(mu) = mom(mu)-k2(mu)
         enddo
      endif
      q1(4) = q1(0)**2-q1(1)**2-q1(2)**2-q1(3)**2
      q2(4) = q2(0)**2-q2(1)**2-q2(2)**2-q2(3)**2

      if (ld0calc) call BCD_fill_diboson(k1,k2,q1,q2)
      d0 = d0c

      d11 = dijc(1,1)
      d12 = dijc(1,2)
      d13 = dijc(1,3)

      d21 = dijc(2,1)
      d22 = dijc(2,2)
      d23 = dijc(2,3)
      d24 = dijc(2,4)
      d25 = dijc(2,5)
      d26 = dijc(2,6)
      d27 = dijc(2,7)

      d31 = dijc(3,1)
      d32 = dijc(3,2)
      d33 = dijc(3,3)
      d34 = dijc(3,4)
      d35 = dijc(3,5)
      d36 = dijc(3,6)
      d37 = dijc(3,7)
      d38 = dijc(3,8)
      d39 = dijc(3,9)
      d310 = dijc(3,10)
      d311 = dijc(3,11)
      d312 = dijc(3,12)
      d313 = dijc(3,13)

Check consistency of kinematics
      if (ldebug) then
         print*," "
         print*," comparison of input to BOXLINE for ivmu = ",ivmu
         print("(a,3f16.4)")," q1sq = ",q1sqc,q1(4),q1sqc/q1(4)
         print("(a,3f16.4)")," q2sq = ",q2sqc,q2(4),q2sqc/q2(4)
         s = -2*dotp(k1,k2)
         t =  2*dotp(k2,q2)+q2(4)
         print("(a,3f16.4)")," s = ",sc,s,sc/s
         print("(a,3f16.4)")," t = ",tc,t,tc/t
c         do mu = 0,3
c            if (abs(k1(mu)-ak1(mu,id)).gt.(1d-10*k1(0)) .or.
c     1          abs(k2(mu)-ak2(mu,id)).gt.(1d-10*k2(0)) .or.
c     1          abs(q1(mu)-aq1(mu,id)).gt.(1d-10*k2(0)) .or.
c     1          abs(q2(mu)-aq2(mu,id)).gt.(1d-10*k2(0)) ) then
c               print*," WARNING: momentum mismatch in boxline, mu =",mu
c               print*," k1: ",k1(mu)/(ak1(mu,id)+1d-80)
c               print*," k2: ",k2(mu)/(ak2(mu,id)+1d-80)
c               print*," q1: ",q1(mu)/(aq1(mu,id)+1d-80)
c               print*," q2: ",q2(mu)/(aq2(mu,id)+1d-80)
c            endif
c         enddo
         print*," input psi1 ",psi1
         print*," input psi2 ",psi2
         print*," input jext ",jext
      endif

c  get matrix elements 
c
c  me1 = psibar(2) eps1slash psi(1)
c  me2 = psibar(2) eps2slash psi(1)
c  mq  = psibar(2) (q1-q2)slash psi(1)
      do mu = 0,3
         if (ivmu.eq.1) then
            do nu = 0,3
               eps1(nu) = 0d0
            enddo
            eps1(mu) = -1d0
            if (mu.eq.0) eps1(mu) = 1d0
         elseif (ivmu.eq.2) then
            do nu = 0,3
               eps2(nu) = 0d0
            enddo
            eps2(mu) = -1d0
            if (mu.eq.0) eps2(mu) = 1d0
         endif
         do isig = -1,isigmax,2
            me1 = s1c(psi2(1,isig),eps1,.true.,isig,psi1(1,isig))
            me2 = s1c(psi2(1,isig),eps2,.true.,isig,psi1(1,isig))
            mq  = 2*s1r(psi2(1,isig),q1,.true.,isig,psi1(1,isig))
            mborn = jborn(mu,isig)
            if (ldebug) then
               print*," check amplitudes for mu =",mu," isig =",isig
               print*," check M_q: ",
     1         -mq/s1r(psi2(1,isig),q2,.true.,isig,psi1(1,isig)),
     1         " = 2?"
               t = mom(0)**2-mom(1)**2-mom(2)**2-mom(3)**2
               z = sc3(psi2(1,isig),eps2,mom,eps1,psi1(1,isig),isig)/t
               print*," M_born:    ",mborn/z," = 1? Abs(ratio) = ",
     &              dreal(mborn/z),abs(mborn/z)
            endif
c
c prepare the coefficients
            s = sc
            t = tc

      eps1k1 = eps1(0)*k1(0)-eps1(1)*k1(1)-eps1(2)*k1(2)-eps1(3)*k1(3)
      eps1k2 = eps1(0)*k2(0)-eps1(1)*k2(1)-eps1(2)*k2(2)-eps1(3)*k2(3)
      eps1q2 = eps1(0)*q2(0)-eps1(1)*q2(1)-eps1(2)*q2(2)-eps1(3)*q2(3)
      eps1eps2 = eps1(0)*eps2(0)-eps1(1)*eps2(1)-
     1           eps1(2)*eps2(2)-eps1(3)*eps2(3)
      eps2k1 = eps2(0)*k1(0)-eps2(1)*k1(1)-eps2(2)*k1(2)-eps2(3)*k1(3)
      eps2k2 = eps2(0)*k2(0)-eps2(1)*k2(1)-eps2(2)*k2(2)-eps2(3)*k2(3)
      eps2q1 = eps2(0)*q1(0)-eps2(1)*q1(1)-eps2(2)*q1(2)-eps2(3)*q1(3)

      eps1q1 = eps1(0)*q1(0)-eps1(1)*q1(1)-eps1(2)*q1(2)-eps1(3)*q1(3)
      eps2q2 = eps2(0)*q2(0)-eps2(1)*q2(1)-eps2(2)*q2(2)-eps2(3)*q2(3)

c
      ce1 = -2*(d12+d24)*eps2k2*(q1sqc+q2sqc-3*s-4*t) - 
     1      2*(d12-d24)*eps2q1*(q2sqc-t) 
     2  +4*(- d11*eps2k2*s - d12*eps2k1*t + d13*eps2k2*
     3  (q2sqc-s-t) + d13*eps2q1*q2sqc - d21*eps2k2*s 
     4 - d22*eps2k2*t - d22*eps2q1*q2sqc + d23*eps2k2*q1sqc
     5 + d25*eps2k2*(q2sqc-s-2*t) 
     6 - d26*eps2k2*(q2sqc-s-t)+d26*eps2q1*t+2*d27*eps2q1
     7 - d32*eps2k2*q2sqc - d34*eps2k2*(q2sqc-t) 
     8 + d35*eps2k2*(q2sqc-s-t)+d36*eps2k2*(2*q2sqc-t) 
     9 + d37*eps2k2*q1sqc + d38*eps2k2*(q1sqc+q2sqc-s)
     & - d39*eps2k2*q1sqc 
     1 - d310*eps2k2*(q1sqc+2*q2sqc-2*s-t)
     2 - 4*d311*eps2k2 + 6*d312*eps2k2 + 2*d313*eps2q1)

      ce2= -2*(d12+d24)*(eps1k2*(q1sqc+q2sqc-s-2*t)
     1  +eps1q2*(q2sqc-s-3*t)) + 
     2  4*(d13*eps1k2*(q1sqc-2*s-t) - 
     3  d13*eps1q2*(2*s+t) + d22*eps1k2*q2sqc
     4  + d22*eps1q2*q2sqc - d23*eps1k2*t + 
     5    d23*eps1q2*(q1sqc-t) - d24*eps1k1*q2sqc 
     6  + d25*eps1k2*q1sqc + d25*eps1k1*(q2sqc-2*s-t) 
     7  + d26*eps1k2*t - d26*eps1k1*(q1sqc-s) - 
     8  2*d27*eps1q2 + d33*eps1k2*q1sqc + d33*eps1q2*q1sqc
     9  + d37*eps1k1*(q2sqc-s-t) + d38*eps1k1*q2sqc
     &  - d39*eps1k1*(q1sqc+q2sqc-s) - d310*eps1k1*(q2sqc-t)
     1  +2*d311*eps1k2+2*d312*eps1q2-6*d313*eps1k1)

C NOTE: cq = 1/2 * cq1 in my reduce code
      cq = (d12+d24)*eps1eps2*s + 2*(4*d12*eps2k2*eps1k2 + 
     1 3*d12*eps2k2*eps1q2+d12*eps2q1*eps1k2-4*d13*eps2k2*eps1k2-
     2 2*d13*eps2k2*eps1q2-2*d13*eps2q1*eps1k2-d13*eps1eps2*s+
     3 2*d22*eps2k2*eps1q2-d22*eps1eps2*t-2*d23*eps2q1*eps1k2-
     4 2*d23*eps2q1*eps1q2-d23*eps1eps2*t+6*d24*eps2k2*eps1k2+
     5 3*d24*eps2k2*eps1q2+d24*eps2q1*eps1k2-6*d25*eps2k2*eps1k2-
     6 2*d25*eps2k2*eps1q2-2*d25*eps2q1*eps1k2-d25*eps1eps2*s-
     7 4*d26*eps2k2*eps1q2+4*d26*eps2q1*eps1k2+2*d26*eps2q1*eps1q2+ 
     8 d26*eps1eps2*(s+2*t)-d32*eps1eps2*q2sqc+
     9 d33*eps1eps2*q1sqc+2*d34*eps2k2*eps1k2-2*d35*eps2k2*eps1k2+
     & 2*d36*eps2k2*eps1q2+d36*eps1eps2*(q2sqc-t)-
     1 2*d37*eps2q1*eps1k2+d37*eps1eps2*(q2sqc-s-t)+
     2 2*d38*eps2q1*eps1q2+d38*eps1eps2*(q1sqc+2*q2sqc-s) -
     3 2*d39*eps2q1*eps1q2-d39*eps1eps2*(2*q1sqc+q2sqc-s) -
     4 2*d310*eps2k2*eps1q2+2*d310*eps2q1*eps1k2-d310*eps1eps2*
     5 (2*q2sqc-s-2*t)+4*d312*eps1eps2-4*d313*eps1eps2)

      cborn = -2*((d36+d37-2*d310)*(q2sqc-t) + 
     1    d38*(q1sqc+2*q2sqc) - d39*(2*q1sqc+q2sqc) )
     2    -2*(d0+d11+d12-2*d13+d24-d25+d26-d37-d38+d39+d310)*s
     3    -2*(-(d22+d23-2*d26)*t + 2*d27 - d32*q2sqc
     4    +d33*q1sqc+ 6*(d312-d313))

      gce1 = -2*eps2q2*( d12*(2*q1sqc+q2sqc-s-3*t) - 
     1       2*(d13-d22)*(q1sqc+q2sqc-s-t)
     2     - d24*(q2sqc-s-t)-2*d26*q1sqc-4*d27-4*d312)

      gce2 = 2*eps1q1*(d12*(q2sqc+t)-2*d13*(q2sqc+s) + 
     1      2*d22*q2sqc + 2*d23*(q1sqc+q2sqc-s-2*t)
     2     +d24*(q2sqc+t)+2*d25*(-q2sqc+s+t)-2*d26*(q2sqc-t)
     3     -4*d27+2*d33*q1sqc + 4*d313)

      gcq = 4*eps1q1*eps2q2*(d12-d13+d22-d23+d38-d39) +
     1        2*eps1q1*eps2k2*(d12-4*d23+d24+2*d26-2*d37+2*d310) -
     2        4*eps1q1*eps2q1*(d23-d26+d33-d39) +
     3        2*eps2q2*eps1k2*(3*d12-4*d13+4*d22+d24-2*d25-
     4                                     2*d26+2*d36-2*d310) +
     5        4*eps2q2*eps1q2*(d12-d13+2*d22-2*d26+d32-d38)
            ce1 = ce1 + gce1
            ce2 = ce2 + gce2
            cq = cq + gcq
c
c  now add the terms from the vertex and propagator corrections
            ce1 = ce1 + 2*eps2k2*Teps2c + 2*eps2q2*Tg2c
            ce2 = ce2 + 2*eps1k1*Teps1c - 2*eps1q1*Tg1c
            cborn = cborn*t - Tbornc

c  and the final result
            mvirt = me1*ce1 + me2*ce2 + mq*cq + mborn*cborn

            if (ldebug) then
               print*," mvirt/mborn ",mvirt/mborn
               print*,"Re(mvirt)/Re(mborn) = ",dreal(mvirt)/dreal(mborn)
               print*,"Im(mvirt)/Im(mborn) = ",dimag(mvirt)/dimag(mborn)
               if (abs(eps1(0)/q1(0)-1).lt.1d-10) then
                  print*," mborn type ",(mborn*cborn+ me2*ce2)/mborn
                  print*," mq type    ",(me1*ce1+ mq*cq)/mborn
                  print*," me1     ",me1*ce1/mborn
                  print*," me2     ",me2*ce2/mborn
               else
                  print*," mborn type ",(mborn*cborn+ me1*ce1)/mborn
                  print*," mq type    ",(me2*ce2+ mq*cq)/mborn
                  print*," me1     ",me1*ce1/mborn
                  print*," me2     ",me2*ce2/mborn
               endif
            endif

c with this additional terms jvirt is gauge invariant and 
c      (-s)^-eps is the prefactor

            jvirt(mu,isig) = mvirt
c     1          + ( 3d0*log(abs(mursq(1,1)/(-sc)))   !-> back to (-s)^-eps
c     2             + (log(abs(mursq(1,1)/(-sc))))**2 !as a prefactor
c     3             + 2d0*(0d0,1d0)*pi*log(abs(mursq(1,1)/(-sc))))
c     4              *jborn(mu,isig)
     5          + (3*pi*(0d0,1d0)-pi**2)*jborn(mu,isig)

         enddo
      enddo

         cq = jvirt(0,-1)*q1(0)-jvirt(1,-1)*q1(1)-
     1        jvirt(2,-1)*q1(2)-jvirt(3,-1)*q1(3)
         ce1= jborn(0,-1)*q1(0)-jborn(1,-1)*q1(1)-
     1        jborn(2,-1)*q1(2)-jborn(3,-1)*q1(3)
         s = abs(cq)/max(abs(jvirt(0,-1)*q1(0)),
     1              abs(jvirt(1,-1)*q1(1)),abs(jvirt(2,-1)*q1(2)),
     2              abs(jvirt(3,-1)*q1(3)))
         t = abs(cq)/max(abs(jborn(0,-1)*q1(0)),
     1              abs(jborn(1,-1)*q1(1)),abs(jborn(2,-1)*q1(2)),
     2              abs(jborn(3,-1)*q1(3)))
         if (t.gt.1d-2.and.ivmu.eq.1) then
c         icount = icount+1
c         if (icount.le.1d5) then

            if (ldebug) then                                           !ldebug
                print*,"s = ",s                                        !ldebug
                print*,"t = ",t                                        !ldebug
         print*," gauge check: ",cq/max(abs(jborn(0,-1)*q1(0)),        !ldebug
     1              abs(jborn(1,-1)*q1(1)),abs(jborn(2,-1)*q1(2)),     !ldebug
     2              abs(jborn(3,-1)*q1(3)))                            !ldebug
         print*," Born check:  ",ce1/max(abs(jborn(0,-1)*q1(0)),       !ldebug
     1              abs(jborn(1,-1)*q1(1)),abs(jborn(2,-1)*q1(2)),     !ldebug
     2              abs(jborn(3,-1)*q1(3)))                            !ldebug
         print*," virt/Born = ", cq/ce1                                !ldebug
         print*," jvirt = ",jvirt(0,-1),jvirt(1,-1),jvirt(2,-1),jvirt(3,-1)
         print*," jborn = ",jborn(0,-1),jborn(1,-1),jborn(2,-1),jborn(3,-1)
         print*,"icount = ",icount                                     !ldebug
            endif                                                      !ldebug

            do mu = 0,3
               do isig = -1,1,2
                  jvirt(mu,isig) = (0d0,0d0)
               enddo
            enddo
c         elseif (icount.gt.1d5) then
c            print*,"stop: too many gauge invariant points in boxlineVV"
c            print*,"icount = ",icount
c            stop
c         endif
      endif

      return
      end

