c****************subroutine QQZZZ *************************************
c
c	Vera Hankele, <vera@particle.physik.uni-karlsruhe.de>
c	Initial version:  2007, September
c	Last modified: 
C
C  QQZZZ calculates the matrix elements**2 for 
C        q1 q2    ---->   Z Z Z,   
C         Z ---> f1-bar f2, Z ---> f3-bar f4, Z ---> f5-bar f6
C
C  and crossing related processes. Pauli interference terms for
c  identical fermions are neglected.
c
C  This code is modified to allow for virtual corrections, more precisely
C  the interference of Born with the finite part of virtual diagrams
C  for 
c
c  INPUT:  NLO = 1       return uuww = |M_born|^2 + 2Re(M_Born^* M_virt)
c          NLO = 0       return uuww = |M_born|^2   etc.
c  for steering the calculation of finite box and pentagon contributions
c  the following additional options are implemented
c          NLO = +4      set all finite box and pentagon contributions to 0
c                        but include cvirt*M_Born contribution
c          NLO = +5      true finite pentagons = 0, all other virtual
c                        contributions included, including leading q^mu/m_V
c                        terms of pentagons which can be related to boxes
c       remaining cases:   return uuww = 2Re(M_born^* M_virt). more precisely
c       the piece which is missing for the full NLO=1 when calling +|NLO|     
c          NLO = -4      finite Box + q^mu/m_V terms of Pentagon contributions
c          NLO = -5      true finite Pentagon contributions only
c
c          N = 1         Momentum configuration 1 (LO)
c          N = 2         Momentum configuration 2: emission of parton 1
c          N = 3         Momentum configuration 3: emission of parton 2
c
c*********************************************************************

      subroutine qqZZZ(pbar,sign, nlo, N, uuzzz, ddzzz, NW)

      implicit none

#include "VBFNLO/utilities/global.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/scales.inc"


c     electroweak couplings are taken from KOPPLN

      double precision  clr, xm2, xmg, b, v, a
      COMMON /BKOPOU/   CLR(4,5,-1:1),XM2(6),XMG(6),B(6,6,6),
     1                  V(4,5),A(4,5)
      double precision  pbar(0:3,2+n_v)
      double precision  uuzzz, ddzzz, res(2),resv(2)
      double precision  p(0:3,2+n_v), fqze(0:4,2), fqzmu(0:4,2)
      double precision  fqztau(0:4,2)
      integer  sign(2+n_v), nlo, mu, i, j, k, isig, N
      integer  ifl(2)
      double complex psize(2,-1:1,2), psiae(2,-1:1,2),
     1               psizmu(2,-1:1,2), psiamu(2,-1:1,2),
     2               psiztau(2,-1:1,2), psiatau(2,-1:1,2)
      double complex jae(0:5,-1:1,2), jam(0:5,-1:1,2),jat(0:5,-1:1,2),
     1               jze(0:5,-1:1,2),jzm(0:5,-1:1,2), jzt(0:5,-1:1,2)
      double complex jvae(0:5,-1:1,2),jvam(0:5,-1:1,2),jvat(0:5,-1:1,2)
      double complex jvze(0:5,-1:1,2),jvzm(0:5,-1:1,2),jvzt(0:5,-1:1,2)
      double complex jvwp(0:5,-1:1,2), jvwm(0:5,-1:1,2)
      double complex j5a1a2(0:3,-1:1,3),j5a2a1(0:3,-1:1,3)
      double complex j5z1z2(0:3,-1:1,3),j5z2z1(0:3,-1:1,3)
      double complex j5a1z2(0:3,-1:1,3),j5a2z1(0:3,-1:1,3)
      double complex j5z1a2(0:3,-1:1,3),j5z2a1(0:3,-1:1,3)
      double complex temp, temp1
      double complex mat(2,-1:1,7), matv(2,-1:1,7)
      double complex madgr(7,-1:1,2)
      double complex mm(2,-1:1), mv(2,-1:1), zero
      double complex ma, mz, m1, m2, m3, m4
      double complex mva, mvz, mv1(3), mv2(3), mv3, mv4
      double complex psi(2,-1:1,2), jqq(0:5,-1:1)
      double complex dotcc, s1c, dotrc
      double precision dotrr, psumsq
      external dotcc, s1c, dotrr, dotrc, psumsq
      save ifl
      parameter (zero = (0d0,0d0) )

c variable for q_mu q_nu terms in the propagators of the massive vector bosons

      double complex zm2i(2:3)
      save zm2i

c  helicity selection

      INTEGER h
      COMMON /hcount / h

c variables for virtual corrections

      double precision scale
      double precision c2,c2o4pi
      parameter (c2=4d0/3d0, c2o4pi=c2/4d0/pi)
      logical lnlo, lbox, lpq, lpt, lpent, ldebug, ldebugm, linit
      data linit /.true./
      save linit
      parameter(ldebugm = .false., ldebug=.false.)

c variables for test purposes

      integer penttrue, pentfalse, boxfalse, boxvar
      double precision deti1, deti2, deti3, deti4, deti5, deti6
      common /Gramdet/ deti1, deti2, deti3, deti4, deti5, deti6
      double precision qmu(0:3), sc, q1sqc, q2sqc, tc, p1p2
      double precision psq, qsq, lsq, tsq, pq, pl, pt, ql, qt, lt, pps
      common /invariants/ pps
      logical veratest
      parameter (veratest=.false.)
      logical gaugetest(-1:1,8), bad_gauge
      save penttrue, pentfalse, boxfalse
      integer NW !NW = 1 means Narrow width approximaion
                 !NW = 0 means full calculation


c---------------------------- BEGIN CODE -----------------------

      bad_gauge = .true.

      lnlo = NLO.ne.0       ! include some virtual stuff if T
      lbox = (NLO.eq.-4) .or. (NLO.eq.1) .or. (NLO.eq.5)    ! call boxline if T
      lpt = (NLO.eq.1) .or. (NLO.eq.-5)   ! true pentagon contribution
c      lpq = (NLO.eq.5) .or. (NLO.eq.-4)   ! q^mu/m_V terms of Pentagon
      lpq = .false.
      lpent = lpt .or. lpq
c define flavors of external quarks for the 2 subprocesses

      if (linit) then
         linit = .false.
         k = 1                  ! uuwwz: u = 3
         ifl(k) = 3
         k = 2                  ! ddwwz: d = 4
         ifl(k) = 4
         penttrue = 0
         pentfalse = 0
         boxfalse = 0
      endif

      do k = 1,2
         do isig = -1,1,2
            do i = 1,7
                  mat(k,isig,i) = (0.0d0,0.0d0)
                  matv(k,isig,i) = (0.0d0,0.0d0)
            enddo
         enddo
      enddo

c define the internal momenta

      do mu = 0,3
         do i = 1,2+n_v
            p(mu,i) = pbar(mu,i)*sign(i)
         enddo
      enddo

c------------------------------------------------

c get the external quark spinors (including factor sqrt(2E) )

      call psi0m(2,pbar(0,1),sign(1),psi)

c get other bra and ket vectors

      do isig =-1,1,2

         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qze(0,N),
     &              ze(1,N),psize(1,isig,2),fqze(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qzmu(0,N),
     &              zmu(1,N),psizmu(1,isig,2),fqzmu(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qztau(0,N),
     &              ztau(1,N),psiztau(1,isig,2),fqztau(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qze(0,N),
     &              ae(1,N),psiae(1,isig,2),fqze(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qzmu(0,N),
     &              amu(1,N),psiamu(1,isig,2),fqzmu(0,2))
         call bra2c(psi(1,isig,2),.true.,p(0,2),isig,qztau(0,N),
     &              atau(1,N),psiatau(1,isig,2),fqztau(0,2))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qze(0,N),
     &              ze(1,N), psize(1,isig,1),fqze(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qzmu(0,N),
     &              zmu(1,N), psizmu(1,isig,1),fqzmu(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qztau(0,N),
     &              ztau(1,N), psiztau(1,isig,1),fqztau(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qze(0,N),
     &              ae(1,N), psiae(1,isig,1),fqze(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qzmu(0,N),
     &              amu(1,N), psiamu(1,isig,1),fqzmu(0,1))
         call ket2c(psi(1,isig,1),.true.,p(0,1),isig,qztau(0,N),
     &              atau(1,N), psiatau(1,isig,1),fqztau(0,1))

      enddo

c Calculation of processes qq -> Z Z Z ---------------------

c calculate pentagon contributions

#ifdef WITH_NLO
      if (lpent) then
         scale = -2d0*dotrr(p(0,1),p(0,2))
         if (NLO.eq.1) then
               do i = 1,3
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ze(1,1),zmu(1,1),zero,zero,i,-5,j5z1z2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  zmu(1,1),ze(1,1),zero,zero,i,-5,j5z2z1(0,-1,i))
                 if (NW.eq.0) then ! all diagrams
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ae(1,1),amu(1,1),zero,zero,i,-5,j5a1a2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  amu(1,1),ae(1,1),zero,zero,i,-5,j5a2a1(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ae(1,1),zmu(1,1),zero,zero,i,-5,j5a1z2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  amu(1,1),ze(1,1),zero,zero,i,-5,j5a2z1(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ze(1,1),amu(1,1),zero,zero,i,-5,j5z1a2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  zmu(1,1),ae(1,1),zero,zero,i,-5,j5z2a1(0,-1,i))
                  endif !NW = 0
               enddo
         elseif (NLO.eq.-5) then
            do i = 1,3
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ze(1,1),zmu(1,1),zero,zero,i,-5,j5z1z2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  zmu(1,1),ze(1,1),zero,zero,i,-5,j5z2z1(0,-1,i))
                  if (NW.eq.0) then ! all diagrams
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ae(1,1),amu(1,1),zero,zero,i,-5,j5a1a2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  amu(1,1),ae(1,1),zero,zero,i,-5,j5a2a1(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ae(1,1),zmu(1,1),zero,zero,i,-5,j5a1z2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  amu(1,1),ze(1,1),zero,zero,i,-5,j5a2z1(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  ze(1,1),amu(1,1),zero,zero,i,-5,j5z1a2(0,-1,i))
            call VVV_J_virtual_tri_box_pent_isig45(psi,p,1,2,sign,scale,
     1                  zmu(1,1),ae(1,1),zero,zero,i,-5,j5z2a1(0,-1,i))
                  endif !NW = 0
            enddo
            call ZZZPentcheck(psi,p,ze(1,1),qze(0,1),zmu(1,1),qzmu(0,1), 
     1         ztau(1,1), qztau(0,1), j5z1z2(0,-1,1)) 
            call ZZZPentcheck(psi, p,zmu(1,1),qzmu(0,1),ze(1,1),qze(0,1),   
     1         ztau(1,1), qztau(0,1), j5z2z1(0,-1,1))
               if (NW.eq.0) then !all diagrams
            call ZZZPentcheck(psi,p,ae(1,1),qze(0,1),amu(1,1),qzmu(0,1),
     1         atau(1,1), qztau(0,1), j5a1a2(0,-1,1))
            call ZZZPentcheck(psi,p,amu(1,1),qzmu(0,1),ae(1,1),qze(0,1), 
     1         atau(1,1), qztau(0,1), j5a2a1(0,-1,1)) 
            call ZZZPentcheck(psi,p,ae(1,1),qze(0,1),zmu(1,1),qzmu(0,1),
     1         atau(1,1), qztau(0,1), j5a1z2(0,-1,1))
            call ZZZPentcheck(psi,p,amu(1,1),qzmu(0,1),ze(1,1),qze(0,1),     
     1         atau(1,1), qztau(0,1), j5a2z1(0,-1,1))
            call ZZZPentcheck(psi,p,ze(1,1),qze(0,1),amu(1,1),qzmu(0,1),
     1         atau(1,1), qztau(0,1), j5z1a2(0,-1,1))
            call ZZZPentcheck(psi,p,zmu(1,1),qzmu(0,1),ae(1,1),qze(0,1), 
     1         atau(1,1), qztau(0,1), j5z2a1(0,-1,1))
               endif !NW = 1
         endif
      endif !lpent

#endif
c   u u -> Z Z Z

      do isig = -1,1,2
         m1=-s1c(psize(1,isig,2),ztau(1,N),.true.,isig,psizmu(1,isig,1))
     1      -s1c(psize(1,isig,2),zmu(1,N),.true.,isig,psiztau(1,isig,1))
     2      -s1c(psizmu(1,isig,2),ztau(1,N),.true.,isig,psize(1,isig,1))
     3      -s1c(psizmu(1,isig,2),ze(1,N),.true.,isig,psiztau(1,isig,1))
     4      -s1c(psiztau(1,isig,2),zmu(1,N),.true.,isig,psize(1,isig,1))
     5      -s1c(psiztau(1,isig,2),ze(1,N),.true.,isig,psizmu(1,isig,1))

         if (NW.eq.0) then
         m2=-s1c(psiae(1,isig,2),atau(1,N),.true.,isig,psiamu(1,isig,1))
     1      -s1c(psiae(1,isig,2),amu(1,N),.true.,isig,psiatau(1,isig,1))
     2      -s1c(psiamu(1,isig,2),atau(1,N),.true.,isig,psiae(1,isig,1))
     3      -s1c(psiamu(1,isig,2),ae(1,N),.true.,isig,psiatau(1,isig,1))
     4      -s1c(psiatau(1,isig,2),amu(1,N),.true.,isig,psiae(1,isig,1))
     5      -s1c(psiatau(1,isig,2),ae(1,N),.true.,isig,psiamu(1,isig,1))

         m3=-s1c(psiae(1,isig,2),ztau(1,N),.true.,isig,psizmu(1,isig,1))
     1      -s1c(psiae(1,isig,2),zmu(1,N),.true.,isig,psiztau(1,isig,1))
     2      -s1c(psiamu(1,isig,2),ztau(1,N),.true.,isig,psize(1,isig,1))
     3      -s1c(psiamu(1,isig,2),ze(1,N),.true.,isig,psiztau(1,isig,1))
     4      -s1c(psiatau(1,isig,2),zmu(1,N),.true.,isig,psize(1,isig,1))
     5      -s1c(psiatau(1,isig,2),ze(1,N),.true.,isig,psizmu(1,isig,1))
     6      -s1c(psize(1,isig,2),atau(1,N),.true.,isig,psizmu(1,isig,1))
     7      -s1c(psize(1,isig,2),amu(1,N),.true.,isig,psiztau(1,isig,1))
     8      -s1c(psizmu(1,isig,2),atau(1,N),.true.,isig,psize(1,isig,1))
     9      -s1c(psizmu(1,isig,2),ae(1,N),.true.,isig,psiztau(1,isig,1))
     &      -s1c(psiztau(1,isig,2),amu(1,N),.true.,isig,psize(1,isig,1))
     1      -s1c(psiztau(1,isig,2),ae(1,N),.true.,isig,psizmu(1,isig,1))
     2      -s1c(psize(1,isig,2),ztau(1,N),.true.,isig,psiamu(1,isig,1))
     3      -s1c(psize(1,isig,2),zmu(1,N),.true.,isig,psiatau(1,isig,1))
     4      -s1c(psizmu(1,isig,2),ztau(1,N),.true.,isig,psiae(1,isig,1))
     5      -s1c(psizmu(1,isig,2),ze(1,N),.true.,isig,psiatau(1,isig,1))
     6      -s1c(psiztau(1,isig,2),zmu(1,N),.true.,isig,psiae(1,isig,1))
     7      -s1c(psiztau(1,isig,2),ze(1,N),.true.,isig,psiamu(1,isig,1))

         m4=-s1c(psiae(1,isig,2),atau(1,N),.true.,isig,psizmu(1,isig,1))
     1      -s1c(psiae(1,isig,2),amu(1,N),.true.,isig,psiztau(1,isig,1))
     2      -s1c(psiamu(1,isig,2),atau(1,N),.true.,isig,psize(1,isig,1))
     3      -s1c(psiamu(1,isig,2),ae(1,N),.true.,isig,psiztau(1,isig,1))
     4      -s1c(psiatau(1,isig,2),amu(1,N),.true.,isig,psize(1,isig,1))
     5      -s1c(psiatau(1,isig,2),ae(1,N),.true.,isig,psizmu(1,isig,1))
     6      -s1c(psize(1,isig,2),atau(1,N),.true.,isig,psiamu(1,isig,1))
     7      -s1c(psize(1,isig,2),amu(1,N),.true.,isig,psiatau(1,isig,1))
     8      -s1c(psizmu(1,isig,2),atau(1,N),.true.,isig,psiae(1,isig,1))
     9      -s1c(psizmu(1,isig,2),ae(1,N),.true.,isig,psiatau(1,isig,1))
     &      -s1c(psiztau(1,isig,2),amu(1,N),.true.,isig,psiae(1,isig,1))
     1      -s1c(psiztau(1,isig,2),ae(1,N),.true.,isig,psiamu(1,isig,1))
     2      -s1c(psiae(1,isig,2),ztau(1,N),.true.,isig,psiamu(1,isig,1))
     3      -s1c(psiae(1,isig,2),zmu(1,N),.true.,isig,psiatau(1,isig,1))
     4      -s1c(psiamu(1,isig,2),ztau(1,N),.true.,isig,psiae(1,isig,1))
     5      -s1c(psiamu(1,isig,2),ze(1,N),.true.,isig,psiatau(1,isig,1))
     6      -s1c(psiatau(1,isig,2),zmu(1,N),.true.,isig,psiae(1,isig,1))
     7      -s1c(psiatau(1,isig,2),ze(1,N),.true.,isig,psiamu(1,isig,1))
         endif !NW = 0

         if (NW.eq.0) then
         mat(1,isig,1) =  m1 * clr(3,2,isig)**3 ! uu -> ZZZ
     1                  + m2 * clr(3,1,isig)**3 ! uu -> AAA
     2                  + m3 * clr(3,2,isig)**2 * clr(3,1,isig) ! uu -> ZZA
     3                  + m4 * clr(3,2,isig) * clr(3,1,isig)**2 ! uu -> AAZ
         mat(2,isig,1) =  m1 * clr(4,2,isig)**3 ! dd -> ZZZ
     1                  + m2 * clr(4,1,isig)**3 ! dd -> AAA
     2                  + m3 * clr(4,2,isig)**2 * clr(4,1,isig) ! dd -> ZZA
     3                  + m4 * clr(4,2,isig) * clr(4,1,isig)**2 ! dd -> AAZ
         else if (NW.eq.1) then
         mat(1,isig,1) =  m1 * clr(3,2,isig)**3 ! uu -> ZZZ
         mat(2,isig,1) =  m1 * clr(4,2,isig)**3 ! dd -> ZZZ
         endif
      enddo

      if (lpent) then
         do isig = -1,1,2
            do i = 1,3
               if (NW.eq.1) then
               mv1(i) = -dotcc(j5z1z2(0,isig,i),ztau(1,N))*clr(3,2,isig)**3
     1                  -dotcc(j5z2z1(0,isig,i),ztau(1,N))*clr(3,2,isig)**3  
               mv2(i) = -dotcc(j5z1z2(0,isig,i),ztau(1,N))*clr(4,2,isig)**3
     1                  -dotcc(j5z2z1(0,isig,i),ztau(1,N))*clr(4,2,isig)**3
               else if (NW.eq.0) then
               mv1(i) = -dotcc(j5a1a2(0,i,isig),atau(1,N))*clr(3,1,isig)**3
     1                  -dotcc(j5a2a1(0,isig,i),atau(1,N))*clr(3,1,isig)**3
     2    -dotcc(j5z1z2(0,isig,i),atau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
     3    -dotcc(j5z2z1(0,isig,i),atau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
     4    -dotcc(j5a1z2(0,isig,i),atau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     5    -dotcc(j5a2z1(0,isig,i),atau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     6    -dotcc(j5z1a2(0,isig,i),atau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     7    -dotcc(j5z2a1(0,isig,i),atau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     8    -dotcc(j5a1a2(0,isig,i),ztau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     9    -dotcc(j5a2a1(0,isig,i),ztau(1,N))*clr(3,2,isig)*clr(3,1,isig)**2
     &    -dotcc(j5z1z2(0,isig,i),ztau(1,N))*clr(3,2,isig)**3
     1    -dotcc(j5z2z1(0,isig,i),ztau(1,N))*clr(3,2,isig)**3  
     2    -dotcc(j5a1z2(0,isig,i),ztau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
     3    -dotcc(j5a2z1(0,isig,i),ztau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
     4    -dotcc(j5z1a2(0,isig,i),ztau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
     5    -dotcc(j5z2a1(0,isig,i),ztau(1,N))*clr(3,1,isig)*clr(3,2,isig)**2
               mv2(i) = -dotcc(j5a1a2(0,isig,i),atau(1,N))*clr(4,1,isig)**3
     1                  -dotcc(j5a2a1(0,isig,i),atau(1,N))*clr(4,1,isig)**3
     2    -dotcc(j5z1z2(0,isig,i),atau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
     3    -dotcc(j5z2z1(0,isig,i),atau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
     4    -dotcc(j5a1z2(0,isig,i),atau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     5    -dotcc(j5a2z1(0,isig,i),atau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     6    -dotcc(j5z1a2(0,isig,i),atau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     7    -dotcc(j5z2a1(0,isig,i),atau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     8    -dotcc(j5a1a2(0,isig,i),ztau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     9    -dotcc(j5a2a1(0,isig,i),ztau(1,N))*clr(4,2,isig)*clr(4,1,isig)**2
     &                  -dotcc(j5z1z2(0,isig,i),ztau(1,N))*clr(4,2,isig)**3
     1                  -dotcc(j5z2z1(0,isig,i),ztau(1,N))*clr(4,2,isig)**3
     2    -dotcc(j5a1z2(0,isig,i),ztau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
     3    -dotcc(j5a2z1(0,isig,i),ztau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
     4    -dotcc(j5z1a2(0,isig,i),ztau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
     5    -dotcc(j5z2a1(0,isig,i),ztau(1,N))*clr(4,1,isig)*clr(4,2,isig)**2
               endif !NW
            enddo
            matv(1,isig,1) = mv1(1)+mv1(2)+mv1(3)
            matv(2,isig,1)=  mv2(1)+mv2(2)+mv2(3)
         enddo
      endif

c Calculation of processes qq -> V V -> Z Z Z -------------

      if (NW.eq.0) then
c qq -> A A/Z (-> Z Z)

      call curr6(1,psi(1,-1,2),p(0,2),
     &           psiae(1,-1,1),fqze(0,1), jae(0,-1,1) )
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psiamu(1,-1,1),fqzmu(0,1), jam(0,-1,1) )
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psiatau(1,-1,1),fqztau(0,1), jat(0,-1,1) )

#ifdef WITH_NLO
      if (lbox) then
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ae(1,N), 2, jae(0,-1,1),
     2                 jvae(0,-1,1) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., amu(1,N), 2, jam(0,-1,1),
     2                 jvam(0,-1,1) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., atau(1,N), 2, jat(0,-1,1),
     2                 jvat(0,-1,1) )
      endif
#endif

c contract with atozz/ztozz tensor

      do isig = -1,1,2
         m1 = -dotcc(azmzt(0,N),jae(0,isig,1))
     1        -dotcc(azezt(0,N),jam(0,isig,1))
     2        -dotcc(azezm(0,N),jat(0,isig,1))
         m2 = -dotcc(zzmzt(0,N),jae(0,isig,1))
     1        -dotcc(zzezt(0,N),jam(0,isig,1))
     2        -dotcc(zzezm(0,N),jat(0,isig,1))
         do k = 1,2
            mat(k,isig,2) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)
         enddo
      enddo

#ifdef WITH_NLO
      if (lbox) then
         do isig = -1,1,2
            mv1(1) = -dotcc(azmzt(0,N),jvae(0,isig,1))
     1            -dotcc(azezt(0,N),jvam(0,isig,1))
     2            -dotcc(azezm(0,N),jvat(0,isig,1))
            mv2(1) = -dotcc(zzmzt(0,N),jvae(0,isig,1))
     1            -dotcc(zzezt(0,N),jvam(0,isig,1))
     2            -dotcc(zzezm(0,N),jvat(0,isig,1))
            do k = 1,2
               matv(k,isig,2) = (mv1(1)*clr(ifl(k),1,isig)
     1                        +  mv2(1)*clr(ifl(k),2,isig))
     2                              *clr(ifl(k),1,isig)
            enddo
         enddo
      endif
#endif

c qq -> A/Z(-> Z Z) A

      call curr6(1,psiae(1,-1,2),fqze(0,2),
     &           psi(1,-1,1),p(0,1), jae(0,-1,2) )
      call curr6(1,psiamu(1,-1,2),fqzmu(0,2),
     &           psi(1,-1,1),p(0,1), jam(0,-1,2) )
      call curr6(1,psiatau(1,-1,2),fqztau(0,2),
     &           psi(1,-1,1),p(0,1), jat(0,-1,2) )

#ifdef WITH_NLO
      if (lbox) then
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ae(1,N), 1, jae(0,-1,2),
     2                 jvae(0,-1,2) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., amu(1,N), 1, jam(0,-1,2),
     2                 jvam(0,-1,2) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., atau(1,N), 1, jat(0,-1,2),
     2                 jvat(0,-1,2) )
      endif
#endif

c contract with atozz/ztozz tensor

      do isig = -1,1,2
         m1 = -dotcc(azmzt(0,N),jae(0,isig,2))
     1        -dotcc(azezt(0,N),jam(0,isig,2))
     2        -dotcc(azezm(0,N),jat(0,isig,2))
         m2 = -dotcc(zzmzt(0,N),jae(0,isig,2))
     1        -dotcc(zzezt(0,N),jam(0,isig,2))
     2        -dotcc(zzezm(0,N),jat(0,isig,2))
         do k = 1,2
            mat(k,isig,3) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),1,isig)
         enddo
      enddo

#ifdef WITH_NLO
      if (lbox) then
         do isig = -1,1,2
            mv1(1) = -dotcc(azmzt(0,N),jvae(0,isig,2))
     1            -dotcc(azezt(0,N),jvam(0,isig,2))
     2            -dotcc(azezm(0,N),jvat(0,isig,2))
            mv2(1) = -dotcc(zzmzt(0,N),jvae(0,isig,2))
     1            -dotcc(zzezt(0,N),jvam(0,isig,2))
     2            -dotcc(zzezm(0,N),jvat(0,isig,2))
            do k = 1,2
               matv(k,isig,3) = (mv1(1)*clr(ifl(k),1,isig)
     1                        +  mv2(1)*clr(ifl(k),2,isig))
     2                              *clr(ifl(k),1,isig)
            enddo
         enddo
      endif
#endif

c qq -> Z A/Z (-> Z Z)

      call curr6(1,psi(1,-1,2),p(0,2),
     &           psize(1,-1,1),fqze(0,1), jze(0,-1,1) )
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psizmu(1,-1,1),fqzmu(0,1), jzm(0,-1,1) )
      call curr6(1,psi(1,-1,2),p(0,2),
     &           psiztau(1,-1,1),fqztau(0,1), jzt(0,-1,1) )

#ifdef WITH_NLO
      if (lbox) then
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ze(1,N), 2, jze(0,-1,1),
     2                 jvze(0,-1,1) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., zmu(1,N), 2, jzm(0,-1,1),
     2                 jvzm(0,-1,1) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ztau(1,N), 2, jzt(0,-1,1),
     2                 jvzt(0,-1,1) )
      endif
#endif

c contract with atozz/ztozz tensor

      do isig = -1,1,2
         m1 = -dotcc(azmzt(0,N),jze(0,isig,1))
     1        -dotcc(azezt(0,N),jzm(0,isig,1))
     2        -dotcc(azezm(0,N),jzt(0,isig,1))
         m2 = -dotcc(zzmzt(0,N),jze(0,isig,1))
     1        -dotcc(zzezt(0,N),jzm(0,isig,1))
     2        -dotcc(zzezm(0,N),jzt(0,isig,1))
         do k = 1,2
            mat(k,isig,4) = (m1*clr(ifl(k),1,isig)
     1                     +  m2*clr(ifl(k),2,isig))
     2                       * clr(ifl(k),2,isig)
         enddo
      enddo

#ifdef WITH_NLO
      if (lbox) then
         do isig = -1,1,2
            mv1(1) = -dotcc(azmzt(0,N),jvze(0,isig,1))
     1            -dotcc(azezt(0,N),jvzm(0,isig,1))
     2            -dotcc(azezm(0,N),jvzt(0,isig,1))
            mv2(1) = -dotcc(zzmzt(0,N),jvze(0,isig,1))
     1            -dotcc(zzezt(0,N),jvzm(0,isig,1))
     2            -dotcc(zzezm(0,N),jvzt(0,isig,1))
            do k = 1,2
               matv(k,isig,4) = (mv1(1)*clr(ifl(k),1,isig)
     1                        +  mv2(1)*clr(ifl(k),2,isig))
     2                              *clr(ifl(k),2,isig)
            enddo
         enddo
      endif
#endif

c qq -> A/Z(-> Z Z) Z

      call curr6(1,psize(1,-1,2),fqze(0,2),
     &           psi(1,-1,1),p(0,1), jze(0,-1,2) )
      call curr6(1,psizmu(1,-1,2),fqzmu(0,2),
     &           psi(1,-1,1),p(0,1), jzm(0,-1,2) )
      call curr6(1,psiztau(1,-1,2),fqztau(0,2),
     &           psi(1,-1,1),p(0,1), jzt(0,-1,2) )

#ifdef WITH_NLO
      if (lbox) then
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ze(1,N), 1, jze(0,-1,2),
     2                 jvze(0,-1,2) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., zmu(1,N), 1, jzm(0,-1,2),
     2                 jvzm(0,-1,2) )
         call TRIboxlinec(1,psi(1,-1,1),psi(1,-1,2),p(0,1),p(0,2),
     1                 .true., ztau(1,N), 1, jzt(0,-1,2),
     2                 jvzt(0,-1,2) )
      endif
#endif

c contract with atozz/ztozz tensor

      do isig = -1,1,2
         m1 = -dotcc(azmzt(0,N),jze(0,isig,2))
     1        -dotcc(azezt(0,N),jzm(0,isig,2))
     2        -dotcc(azezm(0,N),jzt(0,isig,2))
         m2 = -dotcc(zzmzt(0,N),jze(0,isig,2))
     1        -dotcc(zzezt(0,N),jzm(0,isig,2))
     2        -dotcc(zzezm(0,N),jzt(0,isig,2))
         do k = 1,2
            mat(k,isig,5) = (m1*clr(ifl(k),1,isig)
     1                     + m2*clr(ifl(k),2,isig))
     2                         *clr(ifl(k),2,isig)
         enddo
      enddo

#ifdef WITH_NLO
      if (lbox) then
         do isig = -1,1,2
            mv1(1) = -dotcc(azmzt(0,N),jvze(0,isig,2))
     1            -dotcc(azezt(0,N),jvzm(0,isig,2))
     2            -dotcc(azezm(0,N),jvzt(0,isig,2))
            mv2(1) = -dotcc(zzmzt(0,N),jvze(0,isig,2))
     1            -dotcc(zzezt(0,N),jvzm(0,isig,2))
     2            -dotcc(zzezm(0,N),jvzt(0,isig,2))
            do k = 1,2
               matv(k,isig,5) = (mv1(1)*clr(ifl(k),1,isig)
     1                        +  mv2(1)*clr(ifl(k),2,isig))
     2                              *clr(ifl(k),2,isig)
            enddo
         enddo
      endif
#endif
      endif !NW = 0

c  Calculation of processes qq -> A/Z -> Z Z Z ---------------------

      call curr6(1,psi(1,-1,2),p(0,2),psi(1,-1,1),p(0,1),jqq(0,-1))

c contract with atowwz/ztowwz tensor

      do isig = -1,1,2
         if (NW.eq.1) then
            mz = dotcc(zzzz(0,N),jqq(0,isig))
            do k = 1,2
               mat(k,isig,6)  = (0.0d0,0.0d0)
               mat(k,isig,7)  = mz*clr(ifl(k),2,isig)
               matv(k,isig,6) = (0.0d0, 0.0d0)
               matv(k,isig,7) = (0.0d0,0.0d0)
            enddo
         else
            ma = dotcc(azzz(0,N),jqq(0,isig))
            mz = dotcc(zzzz(0,N),jqq(0,isig))
            do k = 1,2
               mat(k,isig,6)  = ma*clr(ifl(k),1,isig)
               mat(k,isig,7)  = mz*clr(ifl(k),2,isig)
               matv(k,isig,6) = (0.0d0, 0.0d0)
               matv(k,isig,7) = (0.0d0,0.0d0)
            enddo
         endif !NW
      enddo

c-----------------------------------------------------
c------------------------------------------------------
c comparison with madgraph

#ifdef WITH_MADGRAPH
      if (ldebugm) then

         do isig = -1,1,2
            do i = 1,7
               do k = 1,2

                  madgr(i,isig,k) = 0d0

               enddo
            enddo
         enddo

         do isig = -1,1,2
            do k = 1,2
               do i =1,7

                  madgr(i,isig,k) = tamp(i,isig,k)

               enddo
            enddo
         enddo

c print results from checks:

         do i=1,7
            do k = 1,2
               do isig = -1,1,2

    	    print*,"ratio for k = ",k," isig = ",isig," i = ",i,":",
     &	    	    mat(k,isig,i)/madgr(i,isig,k)
            print*,"mat(",k,",",isig,",",i,") = ",mat(k,isig,i)
            temp = dreal(mat(k,isig,i))**2+dimag(mat(k,isig,i))**2
            print*,temp
            print*,"madgr(",i,",",isig,",",k,") = ",madgr(i,isig,k)
            temp1 = dreal(madgr(i,isig,k))**2+dimag(madgr(i,isig,k))**2
            print*,temp1
            print*,"temp/temp1 = ",temp/temp1
            print*," "

               enddo
            enddo
         enddo

      endif ! end of debugging block
#endif

c------------------------------------------------------


c sum the graphs, square them and map them onto uuww and ddww

      do k = 1,2
         res(k) = 0
         resv(k) = 0
         do isig = -1,1,2
            mm(k,isig) = 0
            do i = 1,7
               mm(k,isig) = mm(k,isig) +  mat(k,isig,i)
            enddo
            res(k)= res(k) + dreal(mm(k,isig))**2
     &                     + dimag(mm(k,isig))**2

c  add Born type term and multiply by F_q = alphas*C_2/4pi
c  the additional factor for the born term is after adding the 
c  subtraction term and the counter term for the renormalization
c  of the pdfs (see also D(x) in m2s_qqWWZj.f)

            if (lnlo) then
               mv(k,isig)=matv(k,isig,1)+matv(k,isig,2)+matv(k,isig,3)
     1                   +matv(k,isig,4)+matv(k,isig,5)+matv(k,isig,6)
     2                   +matv(k,isig,7)
               if (veratest) then
                  mv(k,isig) = 0d0
               endif
            endif
            if (nlo.gt.0) then
               mv(k,isig) = als(1,1)*c2o4pi*
     1                  (mv(k,isig) + mm(k,isig)* cvirtVV)
            else if(nlo.lt.0) then
               mv(k,isig) = als(1,1)*c2o4pi*mv(k,isig)
            else
                mv(k,isig) = (0d0, 0d0)
            endif

           resv(k) = resv(k) + 2d0*dreal( mm(k,isig)   *
     1              conjg( mv(k,isig) ) )

         enddo

         if (veratest) then
            if (nlo.eq.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif
         else
            if (nlo.ge.0) then
               res(k) = (res(k)+resv(k))*3d0      ! 3 is the color sum factor
            else
               res(k) = resv(k)*3d0               ! 3 is the color sum factor
            endif  
         endif
      enddo

      uuzzz = res(1)
      ddzzz = res(2)

      return
      end

c --------------------------------------------------------------------- 
c --------------------------------------------------------------------- 

#ifdef WITH_NLO

      subroutine J_virtual_tri_box_pent_VVV_isig(psi,p,nq1,nq2,isig,sign, musq,
c     input currents (any ORDERED couple among [J1,J2,J3]) in DHELAS format 
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3 
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3 
     #     num_ret_current,
c     the corresponding Born current  
c           Jborn,                           
c     returned current     
     #     Jret)

      implicit none
      complex * 16 psi(2,-1:1,4)
      double precision p(0:3,8),q(0:3),l(0:3),t(0:3)
      double precision musq
      complex * 16 jal(0:5),jbe(0:5)
      integer num_ret_current, nq1,nq2,nq3,nq4, isig,sign(4)
      complex * 16 jborn(0:3), jret(0:3)
      logical lpr
      common /e0print/lpr

      integer mu
c map input momenta onto incoming standard and make use of
c DHELAS information in currents                
      if (nq1.eq.1 .and. nq2.eq.2) then
         nq3 = 8
         nq4 = 7
      else
         print*," check nq1 etc. in J_virtual_tri_box_pent_VVV_isig call "
         print*,nq1,nq2,nq3,nq4          
         stop    
      endif         
                                                 
      if (num_ret_current.eq.1) then             
         do mu = 0,3                                   
            q(mu) = p(mu,nq3)-p(mu,nq4)               
         enddo                                      
         l(0) = -dreal(jal(4))                   
         l(1) = -dreal(jal(5))                       
         l(2) = -dimag(jal(5))                         
         l(3) = -dimag(jal(4))                     
         t(0) = -dreal(jbe(4))                     
         t(1) = -dreal(jbe(5))                          
         t(2) = -dimag(jbe(5))                          
         t(3) = -dimag(jbe(4))                        
      elseif (num_ret_current.eq.2) then                 
         do mu = 0,3                                     
            l(mu) = p(mu,nq3)-p(mu,nq4)               
         enddo                                      
         q(0) = -dreal(jal(4))                    
         q(1) = -dreal(jal(5))                       
         q(2) = -dimag(jal(5))                     
         q(3) = -dimag(jal(4))                 
         t(0) = -dreal(jbe(4))                       
         t(1) = -dreal(jbe(5))                        
         t(2) = -dimag(jbe(5))                  
         t(3) = -dimag(jbe(4))                          
      elseif (num_ret_current.eq.3) then               
         do mu = 0,3                                   
            t(mu) = p(mu,nq3)-p(mu,nq4)                 
         enddo                                            
         q(0) = -dreal(jal(4))                                   
         q(1) = -dreal(jal(5))                             
         q(2) = -dimag(jal(5))                              
         q(3) = -dimag(jal(4))                               
         l(0) = -dreal(jbe(4))                                       
         l(1) = -dreal(jbe(5))                                            
         l(2) = -dimag(jbe(5))                                            
         l(3) = -dimag(jbe(4))                                        
      endif                                                             
                                                                   
      do mu = 0,3                                                     
         jret(mu) = 0                                             
      enddo                                                            
                                                   
         call  J_virtual_tri_box_pent (                                 
c     bra(p"), ket(p) and helicity
     #     psi(1,-1,nq2),psi(1,-1,nq1),isig,
c     momenta and squared renormalization scale                            
     #     p(0,nq1),q,l,t,p(0,nq2),musq,
c     input currents (any ORDERED couple among [J1,J2,J3])                 
c     They can be  jmu1,jmu2  or  jmu1,jmu3   or   jmu2,jmu3              
     #     jal,jbe,
c     uncontracted external current. It can be 1, 2 or 3               
     #     num_ret_current,
c     returned current                                                  
     #     Jret)

      lpr = .false.
      if (lpr) then
         print*," input momenta for above: p,q,l,t,pp "
         call write_mom(p(0,nq1))
         call write_mom(q)
         call write_mom(l)
         call write_mom(t)
         call write_mom(p(0,nq2))
         print*," "
      endif

c check momentum conservation and subtract Born contribution                   
      do mu = 0,3
         t(mu) = q(mu)+l(mu)+t(mu)+p(mu,nq1)-p(mu,nq2)
         if (abs(t(mu)).gt.1d-8) then
            print*," no momentum conservation in j_virtual_tri_box_pent"
            print*," mom diff(",mu,") = ",t(mu)
         endif
      enddo
      return
      end

#endif
