c---------------- BCD_fill_triboson(k1,k2,q1,q2) --------------------------
c
      subroutine BCD_fill_triboson(k1,k2,q1,q2)
      implicit none
      double precision k1(0:3),k2(0:3),q1(0:3),q2(0:3)
c
c This subroutine does the same as BCD_fill for a single momentum
c assignment as in the diagram
c
c     k1  -->---------->------------- k2
c                 $         $
c                 $         $
c                 $ q1      $  q2
c                 $         $ 
c
c results are stored in Dijc(*,*) etc. i.e.
c
Common block for the output
      complex*16 D0c, Dijc(3,13), Teps1c, Teps2c, Tbornc,Tg1c,Tg2c
      double precision sc, tc, q1sqc, q2sqc
      common /bcd_qqv_c/ sc,tc,q1sqc,q2sqc, D0c,Dijc, 
     1                   Teps1c,Teps2c,Tbornc,Tg1c,Tg2c
c
c local variables, external functions etc.
#include "VBFNLO/utilities/scales.inc"

      double precision p1p2, p1p3, p2p3
      complex*16 C0123,  C0124,  C0134,  C0234
      complex*16 Cij123(2,4), Cij124(2,4), 
     1           Cij134(2,4), Cij234(2,4)
      complex*16 B0a(3), B0aq1, B0aq2, B0at, B0as
      complex*16 B0t, C0t, D0tribos
      external B0t, C0t, D0tribos
      double precision tmq1i, tmq2i
      double precision dotrr, psumsq
      external dotrr, psumsq
      double precision pi,pi2o3
      parameter (pi=3.14159 26535 89793,pi2o3=pi**2/3d0)
c
      sc = -2*dotrr(k1,k2)
      tc = psumsq(k2,q2)
      q1sqc = dotrr(q1,q1)
      q2sqc = dotrr(q2,q2)

c
c now determine the required B0, C0 and D0 functions
c D0 = D0t(s,t,q1sq,q2sq) = D0t(s,t,q2sq,q1sq) with musq = -sc
      D0c = D0tribos(sc,tc,q2sqc,q1sqc,sc)
c
c C0123 = C0(0,q2sq,t), C0124 = C0(0,s,0), C0134 = C0(t,q1sq,0),
c C0234 = C0(q2sq,q1sq,s)
      C0124 = C0t(0d0,sc,0d0,sc)
      C0234 = C0t(q2sqc,q1sqc,sc,sc)
      C0123 = C0t(0d0,q2sqc,tc,sc)
      C0134 = C0t(tc,q1sqc,0d0,sc)
c
c and now the B0 functions: B0t(q1sq), B0t(q2sq), B0t(t)
c Note: B0t(-s,musq=s) = 2
      B0aq1 = B0t(q1sqc,sc)
      B0aq2 = B0t(q2sqc,sc)
      B0at  = B0t(tc,sc)
      B0as  = B0t(sc,sc)
c
c Now everything is ready to calculate the C_ij and D_ij functions
      B0a(1) = 0
      B0a(2) = B0aq2
      B0a(3) = B0at
      call tens3(0d0,0d0,q2sqc,tc,B0a,C0123,Cij123(1,1))
      B0a(1) = 0d0
      B0a(2) = B0as
      B0a(3) = 0d0
      call tens3(0d0,0d0,sc,0d0,B0a,C0124,Cij124(1,1))
      B0a(1) = B0at
      B0a(2) = B0aq1
      B0a(3) = 0d0
      call tens3(0d0,tc,q1sqc,0d0,B0a,C0134,Cij134(1,1))
      B0a(1) = b0aq2
      B0a(2) = B0aq1
      B0a(3) = B0as
      call tens3(0d0,q2sqc,q1sqc,sc,B0a,C0234,Cij234(1,1))
      p1p2 = dotrr(k2,q2)
      p1p3 = dotrr(k2,q1)
      p2p3 = dotrr(q2,q1)
      call tens4( 0d0, 0d0, q2sqc, q1sqc, p1p2, p1p3, p2p3, 
     &            C0123,  C0124,  C0134,  C0234,
     &            Cij123(1,1), Cij124(1,1), 
     &            Cij134(1,1), Cij234(1,1), 
     &            D0c, Dijc(1,1) )
c and the coefficients of the tensors in M_6:
      tmq1i = 1d0/(tc-q1sqc)
      Teps1c = ( (B0at-B0aq1)*(2*tc+3*q1sqc)*tmq1i
     &     +2*B0aq1+1-2*q1sqc*C0134 ) * tmq1i
      Tg1c = (B0aq1-B0at)*tmq1i
c
      tmq2i = 1d0/(tc-q2sqc)
      Teps2c = ( (B0at-b0aq2)*(2*tc+3*q2sqc)*tmq2i
     &     +2*b0aq2+1-2*q2sqc*C0123 ) * tmq2i
      Tg2c = (B0aq2-B0at)*tmq2i
      Tbornc = ( 2*q1sqc*(B0at-B0aq1) + tc*B0at
     &     -q1sqc*B0aq1 )*tmq1i - 2*q1sqc*C0134
     &     + ( 2*q2sqc*(B0at-b0aq2) + tc*B0at
     &     -q2sqc*b0aq2 )*tmq2i - 2*q2sqc*C0123
     &     + B0at - 5d0 + pi2o3
      return
      end


c------------- D0(s,t,q1sq,q2sq)  massless 4-point function -----------
c
      complex*16 function D0tribos(s,t,q1sq,q2sq,musq)
      implicit none
      double precision s, t, q1sq, q2sq, musq
      double precision sl, tl, q1sql, q2sql
      double precision p1sq, p2sq, p3sq, p1p2, p1p3, p2p3

c evaluate the finite part of the scalar 4-point function for zero mass
c propagators 
c  
c  D0 = 1/(i*pi^2) * Int d^dk [-k^2-i0]^-1 [-(k+p1)^2-i0]^-1 
c                      [-(k+p1+p2)^2-i0]^-1 [-(k+p1+p2+p3)^2-i0]^-1
c
c       = pi^-eps Gamma(1+eps) musq^-eps [IR + D0t(s,t,q1sq,q2sq,musq)]
c
c where p1^2 = 0 = (p1+p2+p3)^2 is assumed. The arguments of D0t are
c 
c   s = (p2+p3)^2, t = (p1+p2)^2, q1sq = p3^3 and q2sq = p2^2
c
c IR represents the divergent terms and is given by
c 
c  IR = 1/(s*t)*[1/eps**2 + 1/eps (log(q1sq/t)+log(q2sq/t)+log(musq/-s)) ]
c
c Alternatively, arguments as in the call of tens4 can be used, i.e.
c
c  D0t(s,t,q1sq,q2sq) = D0t4(p1sq, p2sq, p3sq, p1p2, p1p3, p2p3) 
c
c This code agrees with the one written by Carlo. Checked 10/15/2002
c       Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2002 October 5
c	Last modified:    2002 October 14
c  
      complex*16 ipi, ieps, z1, z2, zlog, li2, D0tribos4
      parameter (ipi=(0d0,3.14159 26535 89793),ieps=(0d0,1d-16))
      external li2
      double precision pi,pi2o2,pi2o6
      parameter (pi=3.14159 26535 89793,pi2o2=pi**2/2d0)
      parameter (pi2o6=pi**2/6d0)
      logical ldebug
      parameter (ldebug=.false.)
      double precision rlog, rz1, rz2, rz3
c
      sl = s
      tl = t
      q1sql = q1sq
      q2sql = q2sq
      goto 10
c
      entry D0tribos4(p1sq, p2sq, p3sq, p1p2, p1p3, p2p3, musq)
      if (p1sq.ne.0d0 .or. 
     1    abs(2*(p1p2+p2p3+p1p3)/(p2sq+p3sq)+1).gt.1d-12) then
         print*," Dont use D0t4 for for less than 2 massless momenta "
         print*," (p1p2+p2p3+p1p3)/(p2sq+p3sq) = ",
     1           (p1p2+p2p3+p1p3)/(p2sq+p3sq)
         stop
      endif
      sl = p2sq+2*p2p3+p3sq
      tl = p2sq+2*p1p2
      q1sql = p3sq
      q2sql = p2sq
 10   continue
      if (q1sql.eq.0d0 .or. q2sql.eq.0d0) then
         print*," WARNING: zero Q^2 in D0t: ",q1sql,q2sql
         print*," Set D0t = 0 "
         D0tribos = 0
         return 
      endif
      if (musq.le.0d0) then
         print*," WARNING: non-positive musq in D0t: ",musq
         print*," Set D0t = 0 "
         D0tribos = 0
         return 
      endif
         
c
      rz1 = tl/q1sql
      rz2 = tl/q2sql
      rz3 = -sl/musq
      rlog = -log( abs(rz1*rz2*rz3) )
      zlog = rlog
      if (rz1.gt.0) then
         z1 = 1-rz1
      elseif (tl.lt.0d0) then        !   t<0, q1sq>0
         z1 = 1-rz1-ieps
         zlog = zlog - ipi
      else                           !   t>0, q1sq<0
         z1 = 1-rz1+ieps
         zlog = zlog + ipi
      endif
      if (rz2.gt.0) then
         z2 = 1-rz2
      elseif (tl.lt.0d0) then        !   t<0, q2sq>0
         z2 = 1-rz2-ieps
         zlog = zlog - ipi
      else                           !   t>0, q2sq<0
         z2 = 1-rz2+ieps
         zlog = zlog + ipi
      endif
      if (rz3.lt.0d0) then           !   s>0
         zlog = zlog + ipi
      endif
      D0tribos = (0.5d0*zlog**2 + 2*(li2(z1)+li2(z2)) - pi2o6)/(sl*tl)
c
      if (ldebug) then
         print("(a,2g14.4,a,g14.4)"), 
     1         " t/qsq ratios: ",rz1, rz2," t = ",tl
         print("(a,f8.4,a,f8.4)"),
     1         " zlog = ",dreal(zlog),"+ i pi * ",dimag(zlog)/pi
         z1 = 0.5d0*zlog**2
         rz1 = (dreal(z1)-0.5d0*rlog**2)/pi**2
         if (rlog.ne.0d0) then
            rz2 = dimag(z1)/pi/rlog
         else
            rz2 = 0d0
         endif
         print("(2a,f8.4,a,f8.4,a)")," extra terms from log: ",
     1   " pi**2*(",rz1,")+ i pi rlog *(",rz2,")" 
         print*," D0t = ",d0tribos
      endif
      return 
      end
