c These subroutines compute the matrix element squared for the decays
c h -> W*W* and h-> (Z/gamma)*(Z/gamma)* for
c anomalous hvv couplings
      subroutine m2s_VV(L,bos1,bos2,tau1,tau3,flav1,flav3,m2s)
      implicit none
      real*8 L(0:3,4)           !lepton 4-mom
      real*8 m2s                !matrix element squared
c     
      real*8 colorfac(4)        !colorfac(flav)
      real*8 Q(0:4,2),p13,p24,p32,p14,p12,p34
      real*8 levi       
      real*8 res(0:3)
c      levi = eps-mu-nu-rho-sigma l(mu,1)l(mu,2)l(mu,3)l(mu,4)
c     Q(mu,i) mu is spacetime index
c     i is either 1 or 2
      complex*16 F1,F2,F3,F1c,F2c,F3c,i_
      parameter(i_ = (0.0d0,1.0d0))
c     tau1,tau3 are +1 or -1 chirality
c     bos1,bos2 is boson type 3 for w and 2 for z/gamma
c     flav1,flav3 are flavors of vector boson decay products
      integer tau1,tau3,flav1,flav3,bos1,bos2,mu,i
      complex*16 temp_m2s
c     set up color factors
      colorfac(1) = 1.0d0
      colorfac(2) = 1.0d0
      colorfac(3) = 3.0d0
      colorfac(4) = 3.0d0
c
c     set up dot products 
      p13 = l(0,1)*l(0,3) - l(1,1)*l(1,3) - l(2,1)*l(2,3)- l(3,1)*l(3,3)
      p24 = l(0,2)*l(0,4) - l(1,2)*l(1,4) - l(2,2)*l(2,4)- l(3,2)*l(3,4)
c
      p32 = l(0,2)*l(0,3) - l(1,2)*l(1,3) - l(2,2)*l(2,3)- l(3,2)*l(3,3)
      p14 = l(0,1)*l(0,4) - l(1,1)*l(1,4) - l(2,1)*l(2,4)- l(3,1)*l(3,4)
      p12 = l(0,2)*l(0,1) - l(1,2)*l(1,1) - l(2,2)*l(2,1)- l(3,2)*l(3,1) 
      p34 = l(0,3)*l(0,4) - l(1,3)*l(1,4) - l(2,3)*l(2,4)- l(3,3)*l(3,4)

c     compute levi-civita symbol
      call EPSRRR(res,l(0,2),l(0,3),l(0,4))
      levi = res(0)*l(0,1)-res(1)*l(1,1)-res(2)*l(2,1)-res(3)*l(3,1)

      do mu = 0,3
         Q(mu,1) = l(mu,1) + l(mu,2)
         Q(mu,2) = l(mu,3) + l(mu,4)
      enddo
c     compute qsq for vector bosons
      do i = 1,2
         Q(4,i) = Q(0,i)**2 -  Q(1,i)**2 - Q(2,i)**2 - Q(3,i)**2
      enddo

      call computeFs(Q(4,1),Q(4,2),tau1,tau3,flav1,flav3,
     $     bos1,bos2,F1,F2,F3)

      F1c = conjg(F1)
      F2c = conjg(F2)
      F3c = conjg(F3)

      temp_m2s = f1c*f1 * ( 16*p13*p24 ) +
     $     f1c*f2*(-8*i_*p13*levi+8*i_*p24*levi-8*p13*p14*p32 + 8*p13*
     $   p24**2+8*p13*p12*p34+8*p13**2*p24-8*p14*p32*p24+8*p24*p12*p34)
     $  + f1c*f3 * (8*i_*p13*p14*p32+8*i_*p13*p24**2-8*i_*p13*p12*p34 - 
     $     8*i_*p13**2*p24-8*i_*p14*p32*p24+8*i_*p24*p12*p34-8*p13*levi-
     $     8*p24*levi )
     $     + f2c*f1*(8*i_*p13*levi-8*i_*p24*levi-8*p13*p14*p32+8*p13*
     $   p24**2+8*p13*p12*p34+8*p13**2*p24-8*p14*p32*p24+8*p24*p12*p34)
     $     +f2c*f2*(-32*p13*p14*p32*p24+16*p13**2*p24**2+
     $     16*p13**2*p12*p34-32*p14*p32*p12*p34+16*p14**2*p32**2+
     $     16*p24**2*p12*p34+16*p12**2*p34**2)
     $     +f2c*f3*(-16*i_*p13**2*p12*p34+16*i_*p24**2*p12*p34
     $     -16*p13*p24*levi + 16*p14*p32*levi - 16*p12*p34*levi )
     $     +f3c*f1*(-8*i_*p13*p14*p32-8*i_*p13*p24**2+8*i_*p13*p12*p34+
     $     8*i_*p13**2*p24+8*i_*p14*p32*p24-8*i_*p24*p12*p34-8*p13*levi 
     $     - 8*p24*levi )
     $     + f3c*f2*(16*i_*p13**2*p12*p34-16*i_*p24**2*p12*p34
     $     -16*p13*p24*levi + 16*p14*p32*levi - 16*p12*p34*levi )
     $     + f3c*f3 * ( 32*p13*p14*p32*p24 - 16*p13**2*p24**2 + 
     $     16*p13**2*p12*p34 + 32*p14*p32*p12*p34 - 16*p14**2*p32**2 + 
     $     16*p24**2*p12*p34 - 16*p12**2*p34**2 )
      m2s = dble(temp_m2s) 
c      print*,"temp_m2s=",temp_m2s
      m2s = m2s * colorfac(flav1)*colorfac(flav3)
c      print*,"m2s=",m2s
      return
      end
c     sum overs over chiralities tau1,tau3 =-1,+1
c     
      subroutine m2s_VVsum(L,bos1,bos2,flav1,flav3,m2s)
      implicit none
      integer bos1,bos2,flav1,flav3,i,j
      real*8 L(0:3,4),m2s,temp_m2s

      m2s = 0.0d0
      do i = -1,1,2
         do j = -1,1,2
            call m2s_VV(L,bos1,bos2,i,j,flav1,flav3,temp_m2s)
            m2s = m2s + temp_m2s
         enddo
      enddo
      return
      end

c
C SPATVC calculates the four vector
C   RES-mu = EPS-mu-alpha-beta-rho * A-alpha * B-beta * C-rho
C where EPS is the totally antisymmetric tensor in Bjorken-Drell
C convention. Vector A is assumed complex while B,C are real
C
c      SUBROUTINE SPRRR(RES,A,B,C)
c      IMPLICIT NONE
c      DOUBLE PRECISION  RES(0:3), A(0:3)
c      DOUBLE PRECISION  B(0:3),C(0:3),BC12,BC23,BC31,BC01,BC02,BC03
cc
c      BC12 = B(1)*C(2) - B(2)*C(1)
c      BC23 = B(2)*C(3) - B(3)*C(2)
c      BC31 = B(3)*C(1) - B(1)*C(3)
c      BC01 = B(0)*C(1) - B(1)*C(0)
c      BC02 = B(0)*C(2) - B(2)*C(0)
c      BC03 = B(0)*C(3) - B(3)*C(0)
c      RES(0) = A(1)*BC23 + A(2)*BC31 + A(3)*BC12
c      RES(1) = -A(2)*BC03 + A(3)*BC02 + A(0)*BC23
c      RES(2) = A(1)*BC03 + A(0)*BC31 - A(3)*BC01
c      RES(3) = -A(1)*BC02 + A(2)*BC01 + A(0)*BC12
ccc
c      RETURN
c      END

