c*************************************************************************
      SUBROUTINE InitPhaseSpace
c*************************************************************************
c     Setup phasespace generators.
c*************************************************************************
      implicit none
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjjj,
     1     Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar )
         call def_ps_vnj(6)
      CASE(Hjj_WW,Hjjj_WW)
         call def_ps_vvnj(6,3,4,0,1,1)
      CASE(Hjj_ZZ_ll, Hjj_ZZ_lnu,Hjjj_ZZ_ll,Hjjj_ZZ_lnu)
         call def_ps_vvnj(6,2,2,0,1,1)
      CASE(Zjj_l, Zjj_nu)
         call def_ps_vnj(2)
      CASE(WPjj)
         call def_ps_vnj(3)
      CASE(WMjj)
         call def_ps_vnj(4)
      CASE(WPWMjj)
         call def_ps_vvnj(-6,3,4,1,1,1)
      CASE(ZZjj_ll, ZZjj_lnu) 
         call def_ps_vvnj(-6,2,2,1,1,1)
      CASE(WPZjj)
         if (with_kk) then
         call def_ps_vvnj(-6,3,2,1,1,1)
         else
         call def_ps_vvnj(-6,3,2,1,1,1)
         endif
      CASE(WMZjj)
         if (with_kk) then
         call def_ps_vvnj(-6,4,2,1,1,1)
         else
         call def_ps_vvnj(-6,4,2,1,1,1)
         endif
      CASE(GFHjj)
         call def_ps_vnj(6) 
      CASE(WW)
         call def_ps_qqVVj(3,4)

#ifdef WITH_TRIBOSON
      CASE(WWZ)
         call def_ps_qqVVVj(3,4,2)
      CASE(ZZWP)
         call def_ps_qqVVVj(2,2,3)
      CASE(ZZWM)
         call def_ps_qqVVVj(2,2,4)
      CASE(WWWP)
         call def_ps_WWW(3,4,3)
      CASE(WWWM)
         call def_ps_WWW(4,3,4)
#endif

      CASE DEFAULT
         print*,"No initialization of phase space for this process"
         stop    
      END SELECT

      RETURN
      END


c*************************************************************************
      SUBROUTINE phasespace(rand,
     &                      p, x,
     &                      v,   
     &                      dps  )
c*************************************************************************
c     This is the manager of all phasespace generators and the (only) 
c     interface of those to the main program.  Depending on the process ID
c     the right phasespace generator is called with the correct calling 
c     convention, which might well be individually different.
c
c
c     Currently there is only the phasespace generator of Manuels single top 
c     production in here.
c*************************************************************************
c     INPUT
c     rand            : Array with random numbers driven by monaco. 
c     p         : 4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c                       The first 2 entries are the incoming partons.
c     x               : the feynman x of the incoming partons 
c     v : The 4-momenta of the other particles
c     
c     OUTPUT
c     dps             : The jacobi factor of the phasespace point.
c*************************************************************************
      implicit none
      
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/process.inc"

      real*8 x(nx) 
      real*8 rand(max_PS_dim)
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 dps, dps_dec

      real*8 rn
      real*8 Q(0:4), QV(0:4,2)
      integer i,L,mu
   
      real*8 RandomNumber
      external RandomNumber

c fill momentum arrays with zero
      do L=1,max_kin
         do i = 1,max_p
            do mu=0,3
               p(mu,i,L) = 0d0
            enddo
         enddo
      enddo
      do L=1,max_kin
         do i=1,max_v
            do mu=0,3
               v(mu,i,L) = 0d0
            enddo
         enddo
      enddo

c get additional random number not driven by VEGAS    
      rn = RandomNumber()
c switch between different processes

      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar) ! pp-> Hjj(j) -> xxjj(j)
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)
      CASE(Hjj_WW, Hjj_ZZ_ll, Hjj_ZZ_lnu) ! pp -> Hjj(j) -> VVjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
      CASE(Zjj_l, Zjj_nu, WPjj, WMjj) ! pp -> Vjj(j) -> lljj(j)
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 0)
      CASE(WPWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj) ! pp -> VVjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)
      CASE(HJJJ,Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar) ! pp -> H + 3 jets
         call LIPSN(n_p-nx, rand, rn,
     1              p(0,1,1), p(0,2,1), 
     2              Q, v(0,1,1), v(0,2,1),
     3              p(0,3,1), X(1), X(2), dps, 1)
      CASE(Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu) ! pp -> Hjjj(j) -> VVjjj(j)
         call LIPSNVV(n_p-nx,rand,rn,
     1                p(0,1,1),p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)  
      CASE(GFHjj) ! pp -> Hjj in GF 
         call LIPSN0(n_p-nx, rand, rn,
     1               p(0,1,1), p(0,2,1), 
     2               Q, p(0,3,1), X(1), X(2), dps)
         v(0,1,1) = Q(0)
         v(1,1,1) = Q(1)
         v(2,1,1) = Q(2)
         v(3,1,1) = Q(3)   
      CASE(WW)
         call LIPSVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps)

#ifdef WITH_TRIBOSON
      CASE(WWZ, ZZWP, ZZWM)
         call LIPSVVVJ(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)

      CASE(WWWP,WWWM)
         call LIPSWWW(n_p-nx,rand,rn,
     1                p(0,1,1), p(0,2,1),
     2                v(0,1,1),p(0,3,1),
     3                X(1),X(2),dps,narrowwidth)
#endif

      CASE DEFAULT
         print*," No phase space generator is called"
         stop   
      END SELECT       

c Multiply with symmetry factor for identical leptons in the final state
      dps = dps*symmetryfactor

      RETURN
      END
      


