****************************************************************************
*     This file contains routines for the calculation of the factorization
*     and renormalization scale, alpha_s and momentum transfers.
****************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     SUBROUTINE InitScales
c     SUBROUTINE CalcVBFScales(p, v, Lmax, nlo, bos)
c     SUBROUTINE CalcTopScales(p, v)
c     SUBROUTINE Calc_Momentum_Transfer(p, v, q12, q34, Lmax)
c     FUNCTION alphas5(q2ren,iord1)
c
c***************************************************************************
*   Last modified: 21.05.2007
****************************************************************************

****************************************************************************
      SUBROUTINE InitScales
****************************************************************************
c     Initialization of renormalization and factorization scales
c     and printout for scales.
****************************************************************************

      implicit none

#include "scales.inc"
#include "global.inc"
#include "koppln.inc"
#include "process.inc"

      real*8 qsq
      integer mu
      real*8 alphas5
      external alphas5

      real*8 clr(4,5,-1:1), xm2(6), xmg(6), Bk(6,6,6), Vk(4,5), Ak(4,5)
      COMMON /BKOPOU/ CLR, XM2, XMG, Bk, Vk, Ak


      print*," "
      print*,"Information on scale setting in InitScales "
      print*,"-------------------------------------------"
      print*," "

c-------------------------------      
c   factorization scale
c-------------------------------
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2
         print*,"Constant factorization scale chosen: mu_F =",
     1                   sqrt(qsq)," GeV "
c dynamic scale = momentum transfer of W/Z
      CASE(1)
         SELECT CASE(procID)
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1        Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2        ZZjj_lnu, WPZjj, WMZjj)
            print*,"Dynamical factorization scale chosen:"
            print*,"        momentum transfer of W/Z  "
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_muf = ",id_muf
            STOP
         END SELECT
c dynamic scale = min(pt_j)
      CASE(2)
         SELECT CASE(procID)
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1        Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2        ZZjj_lnu, Hjjj, WPZjj, WMZjj)
            print*,"Dynamical factorization scale chosen:"
            print*,"              mu_F = min(pt_j)"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_muf = ",id_muf
            STOP
         END SELECT
c dynamic scale = invariant VV mass
      CASE(3)
         SELECT CASE(procID)
         CASE(WW)
            print*,"Dynamical factorization scale chosen:"
            print*,"      invariant VV mass "
         CASE DEFAULT
            print *,"unreasonable scale choice for this process:
     1            id_muf = ",id_muf
            STOP
         END SELECT
c dynamic scale = invariant VVV mass
      CASE(4)
         SELECT CASE(procID)
         CASE(ZZWP, ZZWM, WWZ, WWWP, WWWM)
            print*,"Dynamical factorization scale chosen:"
            print*,"      invariant VVV mass "
         CASE DEFAULT
            print *,"unreasonable scale choice for this process:
     1            id_muf = ",id_muf
            STOP
         END SELECT
c dynamic scale = sqrt(pt_j1*pt_j2)
      CASE(5)
         SELECT CASE(procID)
         CASE(GFHjj)
            print*,"Dynamical factorization scale chosen:"
            print*,"              mu_F = sqrt(pt_j1*pt_j2)"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_muf = ",id_muf
            STOP
         END SELECT

      CASE DEFAULT
         print *,"unreasonable scale choice for this process: 
     1         id_muf = ",id_muf
      END SELECT

c-------------------------------      
c   renormalization scale
c-------------------------------
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
         als(1,1) = alphas5(qsq,0)
         print*,"Constant value for alphas chosen: alphas(LO) =",als(1,1)
         als(1,1) = alphas5(qsq,1)
         print*,"Constant value for alphas chosen: alphas(NLO) =",als(1,1)
c dynamic scale = momentum transfer of W/Z
      CASE(1)
         SELECT CASE(procID)
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1        Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2        ZZjj_lnu, WPZjj, WMZjj)
            print*,"Dynamical value for alphas chosen:"
            print*,"alphas = alpha(momentum transfer of W/Z)"   
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_mur = ",id_mur
            STOP
         END SELECT
c dynamic scale alpha(min(pt_j)) 
      CASE(2)
         SELECT CASE(procID)
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1        Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2        ZZjj_lnu, Hjjj, WPZjj, WMZjj)
            print*,"Dynamical scale for alphas chosen:"
            print*,"              alphas = alpha(min(pt_j))"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_mur = ",id_mur
            STOP
         END SELECT
c dynamic scale = invariant VV mass
      CASE(3)
         SELECT CASE(procID)
         CASE(WW)
            print*,"Dynamical value for alphas chosen:"
            print*,"alphas = alpha(invariant VV mass)"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process:
     1            id_mur = ",id_mur
            STOP
         END SELECT
c dynamic scale = invariant VVV mass
      CASE(4)
         SELECT CASE(procID)
         CASE(ZZWP, ZZWM, WWZ, WWWP, WWWM)
            print*,"Dynamical value for alphas chosen:"
            print*,"alphas = alpha(invariant VVV mass)"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process:
     1            id_mur = ",id_mur
            STOP
         END SELECT
c dynamic scale alpha(pt_j1) * alpha(pt_j2) * alpha(m_H)^2
      CASE(5)
         SELECT CASE(procID)
         CASE(GFHjj)
            print*,"Dynamical scale for alphas chosen:"
            print*,"   alphas = alpha(pt_j1)*alpha(pt_j2)* alpha(m_H)^2"
         CASE DEFAULT
            print *,"unreasonable scale choice for this process: 
     1            id_mur = ",id_mur
            STOP
         END SELECT

      CASE DEFAULT
         print *,"unreasonable scale choice for this process: 
     1           id_mur = ",id_mur
         STOP
      END SELECT

      print*," "

      RETURN
      END

c*************************************************************************
      Subroutine Scales(p, v, jets, njets, leptons, nl, photons, 
     1                                        nphotons, Lmax, nlo)
c*************************************************************************
c     This is the general scales routine. It is intended to be the interface
c     to any basic, specialized or user-defined scales routines. 
c*************************************************************************
c     INPUT 
c     p       : 4-momenta of the partons
c     v       : 4-momenta of all other particles (leptons, photons, ...)
c     jets    : the array with the jet information as it is returned by the 
c               jetdefinition routine.
c     njets   : number of jets
c     leptons : the array with the charged lepton information.
c     nl      : number of charged leptons
c     photons : the array with the photon information.
c     nphotons: number of photons
c     Lmax    : number of different kinematics needed
c     nlo     : LO( = 0) or NLO(= 1)
c
c     OUTPUT
c     factorization scale and alfas(ren. scale) is transferred via 
c     common block in scales.inc
c*************************************************************************

      implicit none

#include "scales.inc"
#include "global.inc"
#include "koppln.inc"
#include "process.inc"

c     input variables
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin)     
      real*8 photons(0:7,max_v,max_kin)     
      integer Lmax, njets(max_kin), nl(max_kin), nphotons(max_kin), nlo

      SELECT CASE(procID)

c VBF: Hjj, Vjj, VVjj
      CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW, Hjj_ZZ_ll,
     1     Hjj_ZZ_lnu, Zjj_l, Zjj_nu, WPjj, WMjj, WPWMjj, ZZjj_ll,
     2     ZZjj_lnu, WPZjj, WMZjj)
          call calcVBFscales(p, v, jets, njets, leptons, nl, photons, 
     1                                            nphotons, 3, nlo)

c VBF: Hjjj
      CASE(Hjjj,Hjjj_AA, Hjjj_mu, Hjjj_tau, Hjjj_bbar, Hjjj_WW, 
     1         Hjjj_ZZ_ll,Hjjj_ZZ_lnu)
          call calcH3Jscales(p, v, jets, njets, leptons, nl, photons, 
     1                                            nphotons, Lmax, nlo)

c DIBSON
      CASE(WW)
          call calcVVScales(p, v, nlo, Lmax)

c TRIBOSON
      CASE(WWZ, ZZWP, ZZWM, WWWP, WWWM )
          call calcVVVScales(p, v, nlo, Lmax)

c Gluon fusion
      CASE(GFHjj)
         call calcGGFScales(p)

      CASE DEFAULT
         print *,"Error: scales routine for process "//
     &            "not implemented, ID = ",procID
         STOP
      END SELECT

      RETURN
      END

****************************************************************************
      SUBROUTINE calcVBFscales(p, v, jets, njets, leptons, nl, photons, 
     1                                        nphotons, Lmax, nlo)
****************************************************************************
c     INPUT 
c     p       : 4-momenta of the partons
c     v       : 4-momenta of all other particles (leptons, photons, ...)
c     jets    : the array with the jet information as it is returned by the 
c               jetdefinition routine.
c     njets   : number of jets
c     leptons : the array with the charged lepton information.
c     nl      : number of charged leptons
c     photons : the array with the photon information.
c     nphotons: number of photons
c     Lmax    : number of different kinematics needed
c     nlo     : LO( = 0) or NLO(= 1)
c
c     OUTPUT
c     factorization scale and alfas(ren. scale) is transferred via 
c     common block in scales.inc
****************************************************************************
      implicit none

#include "scales.inc"
#include "global.inc"
#include "koppln.inc"

c     input variables
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin)     
      real*8 photons(0:7,max_v,max_kin)     
      integer Lmax, njets(max_kin), nl(max_kin), nphotons(max_kin), nlo

      real*8 q12(0:4,3), q34(0:4,3), qsq, qsq2, pt(max_jets), exp
      integer L, mu, i
      real*8 alphas5
      external alphas5

c----------------------------      
c   factorization scale
c----------------------------      
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2
         do L = 1,Lmax
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo
c dynamic scale = momentum transfer of W/Z
      CASE(1)
         call Calc_Momentum_Transfer(p, v, q12, q34, Lmax)
         do L = 1,Lmax
            mufsq(1,L) = max(xif**2*q12(4,L),2d0)
            mufsq(2,L) = max(xif**2*q34(4,L),2d0)
         enddo
c dynamic scale = min(pt_j1, pt_j2, ...)
      CASE(2)
         qsq = 1.0d12
         do i = 1,njets(1)
            pt(i) = jets(5,i,1)**2
            pt(i) = MAX(pt(i),4D0)
            qsq = min(qsq,pt(i))
         enddo
         qsq = qsq * xif**2
         do L = 1,Lmax
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo
      CASE DEFAULT
         print *,"unreasonable scale choice : id_muf = ",id_muf
         STOP
      END SELECT

c----------------------------
c   renormalization scale
c----------------------------
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
         if (nlo.eq.0) then
            als(1,1) = alphas5(qsq,0)
         else
            als(1,1) = alphas5(qsq,1)
         endif
         do L = 1,Lmax
            als(1,L) = als(1,1)
            als(2,L) = als(1,1)
         enddo
c dynamic scale = momentum transfer of W/Z
      CASE(1)
         call Calc_Momentum_Transfer(p, v, q12, q34, Lmax)
         do L = 1,Lmax
            qsq = max( xir**2*q12(4,L), 2d0 )
            if (nlo.eq.0) then
               als(1,L) = alphas5( qsq, 0 )
            else
               als(1,L) = alphas5( qsq, 1 )
            endif
            qsq = max( xir**2*q34(4,L), 2d0 )
            if (nlo.eq.0) then
               als(2,L) = alphas5( qsq, 0 )
            else
               als(2,L) = alphas5( qsq, 1 )
            endif
         enddo
c dynamic scale alpha(min(pt_j)) 
      CASE(2)
         qsq = 1.0d12
         do i = 1,njets(1)
            pt(i) = jets(5,i,1)**2
            pt(i) = MAX(pt(i),4D0)
            qsq = min(qsq,pt(i))
         enddo
         qsq = qsq * xir**2
         do L = 1,3
         if (nlo.eq.0) then
            als(1,L) = alphas5(qsq,0)
            als(2,L) = alphas5(qsq,0)
         else
            als(1,L) = alphas5(qsq,1)
            als(2,L) = alphas5(qsq,1)
         endif
         enddo
      CASE DEFAULT
         print *,"unreasonable scale choice : id_mur = ",id_mur
         STOP
      END SELECT

      RETURN
      END

****************************************************************************
      SUBROUTINE calcH3jscales(p, v, jets, njets, leptons, nl, photons, 
     1                                        nphotons, Lmax, nlo)
****************************************************************************
c     INPUT 
c     p       : 4-momenta of the partons
c     v       : 4-momenta of all other particles (leptons, photons, ...)
c     jets    : the array with the jet information as it is returned by the 
c               jetdefinition routine.
c     njets   : number of jets
c     leptons : the array with the charged lepton information.
c     nl      : number of charged leptons
c     photons : the array with the photon information.
c     nphotons: number of photons
c     Lmax    : number of different kinematics needed
c     nlo     : LO( = 0) or NLO(= 1)
c
c     OUTPUT
c     factorization scale and alfas(ren. scale) is transferred via 
c     common block in scales.inc
****************************************************************************
      implicit none

#include "scales.inc"
#include "global.inc"
#include "koppln.inc"

c     input variables
      real*8 p(0:3,max_p,max_kin)
      real*8 v(0:3,max_v,max_kin)
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin)     
      real*8 photons(0:7,max_v,max_kin)     
      integer Lmax, njets(max_kin), nl(max_kin), nphotons(max_kin), nlo

      real*8 q12(0:4,3), q34(0:4,3), qsq, qsq2, pt(max_jets), exp
      integer L, mu, i
      real*8 alphas5
      external alphas5

c----------------------------      
c   factorization scale
c----------------------------      
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2
         do L = 1,Lmax
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo
c dynamic scale = min(pt_j1, pt_j2, ...)
      CASE(2)
         qsq = 1.0d12
         do i = 1,njets(1)
            pt(i) = jets(5,i,1)**2
            pt(i) = MAX(pt(i),4D0)
            qsq = min(qsq,pt(i))
         enddo
         qsq = qsq * xif**2
         do L = 1,Lmax
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo
      CASE DEFAULT
         print *,"unreasonable scale choice : id_muf = ",id_muf
         STOP
      END SELECT

c----------------------------
c   renormalization scale
c----------------------------
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
         if (nlo.eq.0) then
            als(1,1) = alphas5(qsq,0)
         else
            als(1,1) = alphas5(qsq,1)
         endif
         do L = 1,Lmax
            als(1,L) = als(1,1)
            als(2,L) = als(1,1)
         enddo
c dynamic scale alpha(min(pt_j)) 
      CASE(2)
         qsq = 1.0d12
         do i = 1,njets(1)
            pt(i) = jets(5,i,1)**2
            pt(i) = MAX(pt(i),4D0)
            qsq = min(qsq,pt(i))
         enddo
         qsq = qsq * xir**2
         do L = 1,3
         if (nlo.eq.0) then
            als(1,L) = alphas5(qsq,0)
            als(2,L) = alphas5(qsq,0)
         else
            als(1,L) = alphas5(qsq,1)
            als(2,L) = alphas5(qsq,1)
         endif
         enddo
      CASE DEFAULT
         print *,"unreasonable scale choice : id_mur = ",id_mur
         STOP
      END SELECT

      do L = 1, Lmax
         mursq(1,L) = qsq
         mursq(2,L) = qsq
      enddo


      RETURN
      END


****************************************************************************
      SUBROUTINE calcGGFScales(p)
****************************************************************************
c     Calculates factorization and renormalization scales for 
c     single top processes.
****************************************************************************
      implicit none

#include "scales.inc"
#include "global.inc"
#include "koppln.inc"

      real*8 p(0:3,max_p)
      integer l
      real*8 alphas5, qsq, qsq2, pt1, pt2
      external alphas5

      real*8 clr(4,5,-1:1), xm2(6), xmg(6), Bk(6,6,6), Vk(4,5), Ak(4,5)
      COMMON /BKOPOU/ CLR, XM2, XMG, Bk, Vk, Ak

     
c----------------------------      
c   factorization scale
c----------------------------      
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2
         do L = 1,3
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo
c dynamic scale = sqrt(pt_j1*pt_j2)
      CASE(5)
         qsq = p(1,3)**2+p(2,3)**2
         qsq2 = p(1,4)**2+p(2,4)**2
         qsq = MAX(qsq,4D0)
         qsq2 = MAX(qsq2,4D0)
         qsq = sqrt(qsq*qsq2)
         qsq = MAX(qsq,5D0)* xif**2
          do L = 1,3
            mufsq(1,L) = qsq
            mufsq(2,L) = qsq
         enddo

      CASE DEFAULT
         print *,"unreasonable scale choice : id_muf = ",id_muf
         STOP
      END SELECT

c----------------------------
c   renormalization scale
c----------------------------
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
         als(1,1) = alphas5(qsq,0)
         do L = 1,3
            als(1,L) = als(1,1)
            als(2,L) = als(1,1)
            als(3,L) = als(1,1)
         enddo
c dynamic scale alpha(pt_j1) * alpha(pt_j2) * alpha(m_H)^2
      CASE(5)
         PT1=p(1,3)**2+p(2,3)**2
         PT2=p(1,4)**2+p(2,4)**2
         PT1=MAX(PT1,4D0)* xir**2
         PT2=MAX(PT2,4D0)* xir**2
         qsq =  xm2(6) * xir**2
         do L = 1,3
            als(1,L) = alphas5(pt1,0)
            als(2,L) = alphas5(pt2,0)
            als(3,L) = alphas5(qsq,0)
         enddo
       
      CASE DEFAULT
         print *,"unreasonable scale choice : id_mur = ",id_mur
         STOP
      END SELECT


      RETURN
      END

****************************************************************************
      SUBROUTINE CalcVVScales(p, v, nlo, Lmax)
****************************************************************************
c  Calculates factorization and renormalization scales for diboson processes.
c
c  INPUT
c     p    : 4-momenta of the partons involved in the basic process.
c            The first 2 entries are the incoming partons.
c     v    : The 4-momenta of the other particles
c     Lmax : number of momentum configurations, L=1 for direct term
c            L=2,...,Lmax for ptilde momenta for subtraction terms
c     
c  OUTPUT
c     mufsq, mursq, als in scales.inc
****************************************************************************
      implicit none

#include "scales.inc"
#include "global.inc"
#include "coupl.inc"

      integer nlo
      real*8 p(0:3,max_p,max_kin), v(0:3,max_v,max_kin), qsq, qvv(0:4)
      real*8 qv1(0:3), qv2(0:3)
      real*8 mv1, mv2, et1, et2
      integer Lmax, L, mu
      real*8 alphas5, mjj2
      external alphas5, mjj2

      logical ldebug
      parameter (ldebug=.false.)

c----------------------------      
c   factorization scale
c----------------------------      
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2
c dynamical scale = ( invariant VV mass )
      CASE(3)
         do mu = 0,3
            qvv(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
         enddo
         qvv(4) = qvv(0)**2-qvv(1)**2-qvv(2)**2-qvv(3)**2
         qsq = xif**2*(qvv(4))
      CASE DEFAULT
         print*,"in calcvvscales undefined scale choice"
         stop
      END SELECT

      do L = 1,3
         mufsq(1,L) = qsq
         mufsq(2,L) = qsq
      enddo

c----------------------------      
c   renormalization scale
c----------------------------      
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
c dynamical scale = alphas( invariant VV mass )
      CASE(3)
         do mu = 0,3
            qvv(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)+v(mu,4,1)
         enddo
         qvv(4) = qvv(0)**2-qvv(1)**2-qvv(2)**2-qvv(3)**2
         qsq = xir**2*(qvv(4))
c default
      CASE DEFAULT
         print*,"in calcvvscales undefined scale choice "
         stop
      END SELECT

      if (nlo.eq.0) then
         als(1,1) = alphas5(qsq,0)
      else
         als(1,1) = alphas5(qsq,1)
      endif
      do L = 1,3
         mursq(1,L) = qsq
         mursq(2,L) = qsq
         als(1,L) = als(1,1)
         als(2,L) = als(1,1)
      enddo

      RETURN
      END

****************************************************************************
      SUBROUTINE CalcVVVScales(p, v, nlo, Lmax)
****************************************************************************
c  Calculates factorization and renormalization scales for triboson processes.
c
c  INPUT
c     p    : 4-momenta of the partons involved in the basic process.
c            The first 2 entries are the incoming partons.
c     v    : The 4-momenta of the other particles
c     Lmax : number of momentum configurations, L=1 for direct term
c            L=2,...,Lmax for ptilde momenta for subtraction terms
c     
c  OUTPUT
c     mufsq, mursq, als in scales.inc
****************************************************************************
      implicit none

#include "scales.inc"
#include "global.inc"
#include "coupl.inc"

      integer nlo
      real*8 p(0:3,max_p,max_kin), v(0:3,max_v,max_kin), qsq, qvvv(0:4)
      real*8 qv1(0:3), qv2(0:3), qv3(0:3)
      real*8 mv1, mv2, mv3, et1, et2, et3
      integer Lmax, L, mu
      real*8 alphas5, mjj2
      external alphas5, mjj2

      logical ldebug
      parameter (ldebug=.false.)

c----------------------------      
c   factorization scale
c----------------------------      
      SELECT CASE(id_muf)
c user defined constant scale 
      CASE(0)
         qsq = muf_user**2 * xif**2

c dynamical scale = ( invariant VVV mass )
      CASE(4)
         do mu = 0,3
            qvvv(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)
     1                     +v(mu,4,1)+v(mu,5,1)+v(mu,6,1)
         enddo
         qvvv(4) = qvvv(0)**2-qvvv(1)**2-qvvv(2)**2-qvvv(3)**2
         qsq = xif**2*(qvvv(4))
      CASE DEFAULT
         print*,"in calcvvvscales undefined scale choice"
         stop
      END SELECT

      do L = 1,3
         mufsq(1,L) = qsq
         mufsq(2,L) = qsq
      enddo

c----------------------------      
c   renormalization scale
c----------------------------      
      SELECT CASE(id_mur)
c user defined constant scale
      CASE(0)
         qsq = mur_user**2 * xir**2
c dynamical scale = alphas( invariant VVV mass )
      CASE(4)
         do mu = 0,3
            qvvv(mu) = v(mu,1,1)+v(mu,2,1)+v(mu,3,1)
     1                     +v(mu,4,1)+v(mu,5,1)+v(mu,6,1)
         enddo
         qvvv(4) = qvvv(0)**2-qvvv(1)**2-qvvv(2)**2-qvvv(3)**2
         qsq = xir**2*(qvvv(4))
      CASE DEFAULT
         print*,"in calcvvvscales undefined scale choice "
         stop
      END SELECT

      if (nlo.eq.0) then
         als(1,1) = alphas5(qsq,0)
      else
         als(1,1) = alphas5(qsq,1)
      endif
      do L = 1,Lmax
         mursq(1,L) = qsq
         mursq(2,L) = qsq
         als(1,L) = als(1,1)
         als(2,L) = als(1,1)
      enddo

      RETURN
      END

******************************************************************
      SUBROUTINE Calc_Momentum_Transfer(p, v, q12, q34, Lmax)
******************************************************************

      implicit none

#include "global.inc"
      real*8 p(0:3,max_p,max_kin), v(0:3,max_v,max_kin)
      real*8 q12(0:4,3), q34(0:4,3), qsq
      integer Lmax, L, mu
      
      do mu = 0,3
         q12(mu,1) = p(mu,1,1)-p(mu,5,1)-p(mu,3,1)
         q34(mu,1) = p(mu,2,1)-p(mu,5,1)-p(mu,4,1)         
         do L = 2,Lmax
            q12(mu,L) = p(mu,1,L)-p(mu,3,L)
            q34(mu,L) = p(mu,2,L)-p(mu,4,L)
         enddo
      enddo
      do L = 1,Lmax
         q12(4,L) = abs(q12(0,L)**2-q12(1,L)**2-q12(2,L)**2-q12(3,L)**2)
         q34(4,L) = abs(q34(0,L)**2-q34(1,L)**2-q34(2,L)**2-q34(3,L)**2)
      enddo

      RETURN
      END


c****************************************************************************
c     alphas routine, now a wrapper for LHAPDF alphas and hard wired alphas
c     the iord parameter is ignored, when LHAPDF alphas is used.
c****************************************************************************

      DOUBLE PRECISION FUNCTION alphas5(q2ren,iord1)
      IMPLICIT NONE     
      REAL*8 q2ren
      INTEGER iord1
      double precision alphasPDF, alphas5_hardwired
      external alphasPDF, alphas5_hardwired
      
      integer pdflib
      COMMON/PDFparameters/pdflib

      SELECT CASE(pdflib)
      CASE(0)
         alphas5 = alphas5_hardwired(q2ren, iord1)
#ifdef WITH_LHA
      CASE(1)
         alphas5 = alphasPDF(sqrt(q2ren))
#endif
      CASE default
         alphas5 = alphas5_hardwired(q2ren, iord1)
      END SELECT

      RETURN                                                                  
      END       

c=========================================
c===========  alphas5.for ================
c=========================================

      DOUBLE PRECISION FUNCTION alphas5_hardwired(q2ren,iord1)
c same as alphas but for fixed 5 flavors   
      IMPLICIT NONE                                                           
      INTEGER iord1
      REAL*8 q2ren, xnf, lnsc
      REAL*8 dlambda5
      REAL*8 pi
      parameter (pi=3.141592653589793238d0,xnf=5d0)

      LOGICAL*4 ldebug
      parameter (ldebug=.false.)	        !output debug information

      if (iord1.eq.1) then
c         dlambda5=0.2020            !for CTEQ4
         dlambda5=0.2262            !for CTEQ6
      else
c         dlambda5=0.1810            !for CTEQ4
         dlambda5=0.1652            !for CTEQ6
      endif
      if (ldebug) then
         print*," lambda 5 = ", DLAMBDA5
      endif

      IF ( ldebug ) THEN
         WRITE(6,*) "qren",sqrt(q2ren)
         WRITE(6,*) "iord1=",iord1
      ENDIF
       
      lnsc = LOG(q2ren/dlambda5**2)
      IF (iord1 .EQ. 0 ) THEN
         alphas5_hardwired = 12.d0*pi/( (33.d0-2.d0*xnf)*lnsc )   
      ELSEIF (iord1 .EQ. 1) THEN
         alphas5_hardwired = 12.d0*pi/( (33.d0-2.d0*xnf)*lnsc )
     &         *( 1.d0 - 6.d0*(153.d0-19.d0*xnf)/(33.d0-2.d0*xnf)**2
     &                  *LOG(lnsc)/lnsc )
      ELSE
         alphas5_hardwired = 0.1185
      ENDIF                                                      

CASFIX
casfix      alphas5 = 0.1185
CASFIX
      if (ldebug) print*," alphas = ",alphas5_hardwired
      RETURN                                                                  
      END                                                                     
C


c=========================================
c===========  alphas.for =================
c=========================================

      DOUBLE PRECISION FUNCTION alphas(q2ren,xnf,iord1)   
      IMPLICIT NONE                                                           
      INTEGER iord1
      REAL*8 q2ren,xnf
      REAL*8 dlambda3,dlambda5,dlambda6,dlambda1
      REAL*8 pi
      parameter (pi=3.141592653589793238d0)
      REAL*8 amc,amb,amt
      parameter (amc=1.35d0,amb=4.5d0,amt=175.0d0)       

      LOGICAL*4 ldebug
c---------------------------
      REAL*8 DLAMBDA4
      COMMON/QCDLAM/DLAMBDA4
c---------------------------
      parameter (ldebug=.true.)	        !output debug information
      if (iord1.eq.1) then
         dlambda4=0.325
      else
         dlambda4=0.215
      endif
      if (ldebug) then
         print*," lambda 4 = ", DLAMBDA4," nf = ",xnf
      endif

      IF (iord1. eq. 0) THEN                                                  
         dlambda3=dlambda4*(amc/dlambda4)**(2./27.)                           
         dlambda5=dlambda4*(dlambda4/amb)**(2./23.)                           
         dlambda6=dlambda5*(dlambda5/amt)**(2./21.)                           
      ELSEIF (iord1. eq. 1) THEN
         dlambda3=dlambda4*(amc/dlambda4)**(2./27.)                           
     &           *(dlog(amc**2/dlambda4**2))**(107./2025)                     
         dlambda5=dlambda4*(dlambda4/amb)**(2./23.)                           
     &           *(dlog(amb**2/dlambda4**2))**(-963./13225.)                  
         dlambda6=dlambda5*(dlambda5/amt)**(2./21.)                           
     &           *(dlog(amt**2/dlambda5**2))**(-321./3381.)                   
      ENDIF                                                                   

      IF (xnf .EQ. 4) THEN                                                   
         dlambda1=dlambda4                                                    
      ELSEIF (xnf .EQ. 5) THEN                                               
         dlambda1=dlambda5                                                    
      ELSEIF (xnf .EQ. 6) THEN   
         dlambda1=dlambda6                                   
      ELSE
         WRITE(6,*)"warning: xnf not defined in alphas"
         STOP
      ENDIF                                                                   

      IF ( ldebug ) THEN
         WRITE(6,*) "lambda_3=",dlambda3                                      
         WRITE(6,*) "lambda_4=",dlambda4                                      
         WRITE(6,*) "lambda_5=",dlambda5                                      
         WRITE(6,*) "lambda_6=",dlambda6                                      
         WRITE(6,*) "dlambda1",dlambda1
         WRITE(6,*) "q2ren",q2ren
         WRITE(6,*) "xnf",xnf
         WRITE(6,*) "iord1=",iord1
       ENDIF
       
       IF (iord1 .EQ. 0 ) THEN
          alphas = 12.d0*pi/( (33.d0-2.d0*xnf)*LOG(q2ren/dlambda1**2) )   
       ELSEIF (iord1 .EQ. 1) THEN
          alphas = 12.d0*pi/( (33.d0-2.d0*xnf)*LOG(q2ren/dlambda1**2) )
     &         *( 1.d0 - 6.d0*(153.d0-19.d0*xnf)/(33.d0-2.d0*xnf)**2
     &                  *LOG(LOG(q2ren/dlambda1**2))
     &                  /LOG(q2ren/dlambda1**2)               )
      ELSE
         alphas = 0.1185
      ENDIF                                                      

CASFIX
casfix      alphas = 0.1185
CASFIX
      RETURN                                                                  
      END                                                                     
